# -*- coding: utf-8 -*-

from ecl.network.v2 import common_function_pool as _common_function_pool
from ecl.network.v2 import common_function as _common_function
from ecl.network.v2 import common_function_gateway as _common_function_gateway
from ecl.network.v2 import colocation_space as _colocation_space
from ecl.network.v2 import colocation_physical_link as _colocation_physical_link
from ecl.network.v2 import colocation_logical_link as _colocation_logical_link
from ecl.network.v2 import extension as _extension
from ecl.network.v2 import network as _network
from ecl.network.v2 import subnet as _subnet
from ecl.network.v2 import port as _port
from ecl.network.v2 import physical_port as _physical_port
from ecl.network.v2 import quota as _quota
from ecl.network.v2 import reserved_address as _reserved_address
from ecl.network.v2 import firewall as _firewall
from ecl.network.v2 import firewall_interface as _firewall_if
from ecl.network.v2 import firewall_plan as _firewall_plan
from ecl.network.v2 import firewall_action as _firewall_action
from ecl.network.v2 import load_balancer as _load_balancer
from ecl.network.v2 import load_balancer_interface as _load_balancer_if
from ecl.network.v2 import load_balancer_plan as _load_balancer_plan
from ecl.network.v2 import load_balancer_action as _load_balancer_action
from ecl.network.v2 import load_balancer_syslog_server as _load_balancer_syslog
from ecl.network.v2 import gw_interface as _gwif
from ecl.network.v2 import internet as _internet
from ecl.network.v2 import publicip as _publicip
from ecl.network.v2 import qos_option as _qos_option
from ecl.network.v2 import static_route as _static_route
from ecl.network.v2 import vpn as _vpn
from ecl.network.v2 import interdc as _interdc
from ecl.network.v2 import aws as _aws
from ecl.network.v2 import gcp as _gcp
from ecl.network.v2 import tenant_connection as _tenant_connection
from ecl.network.v2 import azure as _azure

from ecl import proxy2


class Proxy(proxy2.BaseProxy):

    def create_network(self, admin_state_up=None, description=None,
                       name=None, plane=None, tenant_id=None,
                       tags=None):
        """Create a new network from attributes

        :param bool admin_state_up: administrative state,
                                     default true
        :param string description: description of network
        :param string name: name of network
        :param string plane: Type of the traffic will be used.
                           value should be "data" or "storage"
        :param string tenant_id: tenant id to create network
        :param dict tags: tags of network

        :returns: The results of network creation
        :rtype: :class:`~ecl.network.v2.network.Network`
        """
        body = dict()
        body.setdefault("admin_state_up", False)

        if admin_state_up:
            body["admin_state_up"] = admin_state_up
        if description:
            body["description"] = description
        if name:
            body["name"] = name
        if tenant_id:
            body["tenant_id"] = tenant_id
        if plane:
            body["plane"] = plane
        if tags:
            body["tags"] = tags
        return self._create(_network.Network, **body)

    def delete_network(self, network, ignore_missing=False):
        """Delete a network

        :param network:
            The value can be either the ID of a network or a
            :class:`~ecl.network.v2.network.Network` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the network does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent network.

        :returns: ``None``
        """
        self._delete(_network.Network, network, ignore_missing=ignore_missing)

    def find_network(self, name_or_id, ignore_missing=False):
        """Find a single network

        :param name_or_id: The name or ID of a network.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.network.Network` or None
        """
        return self._find(_network.Network, name_or_id,
                          ignore_missing=ignore_missing)

    def get_network(self, network):
        """Get a single network

        :param network:
            The value can be the ID of a network or a
            :class:`~ecl.network.v2.network.Network` instance.

        :returns: One :class:`~ecl.network.v2.network.Network`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_network.Network, network)

    def networks(self, **params):
        """Return a list of networks
        :param params: The parameters as query string
                       to get networks by specified condition.
        :returns: A list of network objects
        :rtype: list of :class:`~ecl.network.v2.network.Network`
        """
        return list(self._list(_network.Network, paginated=False, **params))

    def update_network(self, network, **params):
        """Update a network

        :param network:
            Either the id of a network or a
            :class:`~ecl.network.v2.network.Network` instance.
        :attrs \*\*params: Parameters for network update.

            * bool admin_state_up: admin state of network to update
            * string name: name of network to update
            * string description: description of network to update
            * dict tags: tags of network to update

        :returns: The updated network
        :rtype: :class:`~ecl.network.v2.network.Network`
        """
        if not isinstance(network, _network.Network):
            network = self._get_resource(_network.Network, network)
            network._body.clean()
        return self._update(_network.Network, network, **params)

    def find_extension(self, name_or_id, ignore_missing=False):
        """Find a single extension

        :param name_or_id: The name or ID of a extension.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.extension.Extension`
                  or ``None``
        """
        return self._find(_extension.Extension, name_or_id,
                          ignore_missing=ignore_missing)

    def extensions(self):
        """Return a list of extensions

        :returns: A list of extension objects
        :rtype: :class:`~ecl.network.v2.extension.Extension`
        """
        return list(self._list(_extension.Extension, paginated=False))

    def get_extension(self, extension):
        """Get a single extension

        :param extension:
            The value can be the ID of a extension or a
            :class:`~ecl.network.v2.extension.Extension` instance.

        :returns: One :class:`~ecl.network.v2.extension.Extension`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_extension.Extension, extension)

    def create_port(self, admin_state_up=None, allowed_address_pairs=None,
                    mac_address=None, description=None, device_id=None,
                    device_owner=None, fixed_ips=None, name=None,
                    network_id=None, segmentation_id=None,
                    segmentation_type=None, tags=None):
        """Create a new port from attributes

        :param string admin_state_up: The admin state of port to create
        :param array allowed_address_pairs: allowed address pairs
                        e.g. [{"mac_address": <mac>, "ip_address": <ipv4/cidr>}]
        :param string mac_address: The mac address of port to create
        :param string description: The description of port to create
        :param string device_id: The device id of port to create
        :param string device_owner: The device owner of port to create
        :param array fixed_ips: The fixed ips of port to create
                        e.g. [{"ip_address":<ipv4> , "subnet_id": <uuid> }, ]
        :param string name: The name of port to create
        :param string network_id: The network id of port to create
        :param int segmentation_id: The segmentation id of port to create
        :param string segmentation_type: The segmentation type of port to create
        :param dict tags: tags of port

        :returns: The results of port creation
        :rtype: :class:`~ecl.network.v2.port.Port`
        """
        body = dict()
        if admin_state_up:
            body["admin_state_up"] = admin_state_up
        if allowed_address_pairs:
            body["allowed_address_pairs"] = allowed_address_pairs
        if mac_address:
            body["mac_address"] = mac_address
        if description:
            body["description"] = description
        if device_id:
            body["device_id"] = device_id
        if device_owner:
            body["device_owner"] = device_owner
        if fixed_ips or fixed_ips == []:
            body["fixed_ips"] = fixed_ips
        if name:
            body["name"] = name
        if network_id:
            body["network_id"] = network_id
        if segmentation_id:
            body["segmentation_id"] = segmentation_id
        if segmentation_type:
            body["segmentation_type"] = segmentation_type
        if tags:
            body["tags"] = tags

        return self._create(_port.Port, **body)

    def delete_port(self, port, ignore_missing=False):
        """Delete a port

        :param port: The value can be either the ID of a port or a
                     :class:`~ecl.network.v2.port.Port` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the port does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent port.

        :returns: ``None``
        """
        self._delete(_port.Port, port, ignore_missing=ignore_missing)

    def find_port(self, name_or_id, ignore_missing=False):
        """Find a single port

        :param name_or_id: The name or ID of a port.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.port.Port` or None
        """
        return self._find(_port.Port, name_or_id,
                          ignore_missing=ignore_missing)

    def get_port(self, port):
        """Get a single port

        :param port: The value can be the ID of a port or a
                     :class:`~ecl.network.v2.port.Port` instance.

        :returns: One :class:`~ecl.network.v2.port.Port`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_port.Port, port)

    def ports(self, **query):
        """Return a list of ports

        :param query: Query parameters to select results
        :returns: A list of port objects
        :rtype: :class:`~ecl.network.v2.port.Port`
        """
        return list(self._list(_port.Port, paginated=False, **query))

    def update_port(self, port, **params):
        """Update a port

        :param port: Either the id of a port or a
                     :class:`~ecl.network.v2.port.Port` instance.
        :param kwargs \*\*params: Parameters for Port update.

            * string admin_state_up: The admin state of port to update
            * array allowed_address_pairs: allowed address pairs
                e.g. [{"mac_address": <mac>, "ip_address": <ipv4/cidr>}]
            * string mac_address: The mac address of port to update
            * string description: The description of port to update
            * string device_id: The device id of port to update
            * string device_owner: The device owner of port to update
            * array fixed_ips: The fixed ips of port to update
                        e.g. [{"ip_address":<ipv4> , "subnet_id": <uuid> }, ]
            * string name: The name of port to update
            * int segmentation_id: The segmentation id of port to update
            * string segmentation_type: The segmentation type of port to update
            * dict tags: tags of port

        :returns: The updated port
        :rtype: :class:`~ecl.network.v2.port.Port`
        """
        if not isinstance(port, _port.Port):
            port = self._get_resource(_port.Port, port)
            port._body.clean()
        return self._update(_port.Port, port, **params)

    def get_physical_port(self, physical_port):
        """Get a single physical_port

        :param physical_port: The value can be the ID of a physical_port or a
                     :class:`~ecl.network.v2.physical_port.PhysicalPort` instance.

        :returns: One :class:`~ecl.network.v2.port.Port`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_physical_port.PhysicalPort, physical_port)

    def physical_ports(self, **query):
        """
        List all visible physical_ports.

        :param query: Query parameters to select results
        :return: A generator of physical_ports.
        :rtype: :class:`~ecl.network.v2.physical_port.PhysicalPort`
        """
        return list(self._list(_physical_port.PhysicalPort, **query))

    def get_quota(self, quota):
        """Get a quota

        :param quota: The value can be the ID of a quota or a
                      :class:`~ecl.network.v2.quota.Quota` instance.
                      The ID of a quota is the same as the tenant ID
                      for the quota.

        :returns: One :class:`~ecl.network.v2.quota.Quota`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_quota.Quota, quota)

    def create_subnet(self, network_id, cidr, allocation_pools=None,
                      description=None, dns_nameservers=None, enable_dhcp=None,
                      gateway_ip=None, host_routes=None, ip_version=4, name=None,
                      ntp_servers=None, tags=None):
        """Create a new subnet from attributes

        :param string network_id: ID of network that subnet belongs to.
        :param CIDR cidr: cidr of subnet to create
        :param array allocation_pools: allocation pools of subnet to create
        :param string description: description of subnet to create
        :param array dns_nameservers: dns nameservers of subnet to create
        :param bool enable_dhcp: bool value indicating if dhcp is enabled
        :param IPv4 gateway_ip: Gateway IP of subnet to create
        :param array host_routes: host routes of subnet to create
        :param Integer ip_version: support v4 only
        :param string name: name of subnet to create
        :param array ntp_servers: ntp servers of subnet to create
        :param dict tags: tags of subnet to create

        :returns: The results of subnet creation
        :rtype: :class:`~ecl.network.v2.subnet.Subnet`
        """

        body = {
            "network_id": network_id,
            "cidr": cidr,
            "enable_dhcp": enable_dhcp,
            "gateway_ip": gateway_ip
        }

        if allocation_pools:
            body.update({"allocation_pools": allocation_pools})
        if description:
            body.update({"description": description})
        if dns_nameservers:
            body.update({"dns_nameservers": dns_nameservers})
        if host_routes:
            body.update({"host_routes": host_routes})
        if ip_version:
            body.update({"ip_version": ip_version})
        if name:
            body.update({"name": name})
        if ntp_servers:
            body.update({"ntp_servers": ntp_servers})
        if tags:
            body.update({"tags": tags})

        return self._create(_subnet.Subnet, **body)

    def delete_subnet(self, subnet, ignore_missing=False):
        """Delete a subnet

        :param subnet: The value can be either the ID of a subnet or a
                       :class:`~ecl.network.v2.subnet.Subnet` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the subnet does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent subnet.

        :returns: ``None``
        """
        self._delete(_subnet.Subnet, subnet, ignore_missing=ignore_missing)

    def find_subnet(self, name_or_id, ignore_missing=False):
        """Find a single subnet

        :param name_or_id: The name or ID of a subnet.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.subnet.Subnet` or None
        """
        return self._find(_subnet.Subnet, name_or_id,
                          ignore_missing=ignore_missing)

    def get_subnet(self, subnet):
        """Get a single subnet

        :param subnet: The value can be the ID of a subnet or a
                       :class:`~ecl.network.v2.subnet.Subnet` instance.

        :returns: One :class:`~ecl.network.v2.subnet.Subnet`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_subnet.Subnet, subnet)

    def subnets(self, **query):
        """Return a list of subnets

        :param query: Query parameters to select results
        :returns: A list of subnet objects
        :rtype: :class:`~ecl.network.v2.subnet.Subnet`
        """
        return list(self._list(_subnet.Subnet, paginated=False, **query))

    def update_subnet(self, subnet, **params):
        """Update a subnet

        :param subnet: Either the id of a subnet or a
                       :class:`~ecl.network.v2.subnet.Subnet` instance.
        :param \*\*params: Parameters for subnet update.

            * string description: description of subnet to create
            * array dns_nameservers: dns nameservers of subnet to create
            * bool enable_dhcp: bool value indicating if dhcp is enabled
            * IPv4 gateway_ip: Gateway IP of subnet to create
            * array host_routes: host routes of subnet to create
            * string name: name of subnet to create
            * array ntp_servers: ntp servers of subnet to create
            * dict tags: tags of subnet to create

        :returns: The updated subnet
        :rtype: :class:`~ecl.network.v2.subnet.Subnet`
        """
        if not isinstance(subnet, _subnet.Subnet):
            subnet = self._get_resource(_subnet.Subnet, subnet)
            subnet._body.clean()

        return self._update(_subnet.Subnet, subnet, **params)

    def reserved_addresses(self, **query):
        """Return a list of reserved addresses

        :returns: A list of reserved addresses objects
        :rtype: list of :class:`~ecl.network.v2.reserved_address.ReservedAddress`
        """
        return list(self._list(_reserved_address.ReservedAddress,
                               aginated=False, **query))

    def get_reserved_address(self, reserved_address):
        """Get a reserved address

        :param reserved_address: The value can be the ID of a reserved address or a
                      :class:`~ecl.network.v2.reserved_address.ReservedAddress` instance.

        :returns: One :class:`~ecl.network.v2.reserved_address.ReservedAddress`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_reserved_address.ReservedAddress, reserved_address)

    def firewalls(self, **query):
        """
        List all visible firewalls.

        :param query: Query parameters to select results
        :return: A generator of firewall instances.
        :rtype: :class:`~ecl.network.v2.firewall.Firewall`
        """
        return list(self._list(_firewall.Firewall, **query))

    def create_firewall(self, firewall_plan_id, availability_zone=None,
                        default_gateway=None, description=None, name=None):
        """
        Create firewall.

        :param firewall_plan_id: Firewall Plan ID.
        :param availability_zone: Get from
            `~ecl.compute.v2.availability_zone.AvailabilityZone`
        :param default_gateway: IP address of default gateway.
        :param description: Description of the Firewall.
        :param name: Name of the Firewall.
        :return: :class:`~ecl.network.v2.firewall.Firewall`
        """
        body = {"firewall_plan_id": firewall_plan_id}
        if availability_zone:
            body["availability_zone"] = availability_zone
        if default_gateway:
            body["default_gateway"] = default_gateway
        if description:
            body["description"] = description
        if name:
            body["name"] = name
        return self._create(_firewall.Firewall, **body)

    def get_firewall(self, firewall_id):
        """
        Get details for firewall.

        :param firewall_id: ID of specified firewall.
        :return: :class:`~ecl.network.v2.firewall.Firewall`
        """
        return self._get(_firewall.Firewall, firewall_id)

    def update_firewall(self, firewall_id, **params):
        """
        Update a specified firewall.

        :param firewall_id: ID of specified firewall.
        :param \*\*params: Parameters for firewall update.

            * default_gateway: IP address of default gateway.
            * description: Description of the Firewall.
            * firewall_plan_id: Firewall Plan ID.
            * name: Name of the Firewall.

        :return: :class:`~ecl.network.v2.firewall.Firewall`
        """
        firewall = _firewall.Firewall()
        return firewall.update(self.session, firewall_id, **params)

    def delete_firewall(self, firewall_id):
        """
        Delete a sepcified firewall.

        :param firewall_id: ID of specified firewall.
        :return: ``None``
        """
        return self._delete(_firewall.Firewall, firewall_id)

    def find_firewall(self, name_or_id, ignore_missing=False):
        """Find a single firewall

        :param name_or_id: The name or ID of a firewall.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.firewall.Firewall` or None
        """
        return self._find(_firewall.Firewall, name_or_id,
                          ignore_missing=ignore_missing)

    def firewall_interfaces(self, **query):
        """
        List all visible firewall_interfaces.

        :param query: Query parameters to select results
        :return: A generator of FirewallInterface instances.
        :rtype: :class:`~ecl.network.v2.firewall_interface.FirewallInterface`
        """
        return list(self._list(_firewall_if.FirewallInterface, **query))

    def get_firewall_interface(self, firewall_interface_id):
        """
        Get details for firewall_interface.

        :param firewall_interface_id: ID of specified firewall_interface.
        :return: :class:`~ecl.network.v2.firewall_interface.FirewallInterface`
        """
        return self._get(_firewall_if.FirewallInterface,
                         firewall_interface_id)

    def update_firewall_interface(self, firewall_id,
                                  firewall_interface_id,
                                  **params):
        """
        Update firewall_interface.

        :param firewall_id: ID of parent firewall ID of firewall_interface.
        :param firewall_interface_id: ID of specified firewall_interface.
        :param params: Parameters for update.
        :return: :class:`~ecl.network.v2.firewall_interface.FirewallInterface`
        """
        interface = _firewall_if.FirewallInterface()
        return interface.update(self.session, firewall_id,
                                firewall_interface_id, **params)

    def firewall_plans(self, **query):
        """
        List all visible firewall_plans.

        :param query: Query parameters to select results
        :return: A generator of FirewallPlan instances.
        :rtype: :class:`~ecl.network.v2.firewall_plan.FirewallPlan`
        """
        return list(self._list(_firewall_plan.FirewallPlan, **query))

    def get_firewall_plan(self, firewall_plan_id):
        """
        Get details for firewall_plan.

        :param firewall_plan_id: Specified firewall plan id.
        :return: :class:`~ecl.network.v2.firewall_plan.FirewallPlan`
        """
        return self._get(_firewall_plan.FirewallPlan, firewall_plan_id)

    def find_firewall_plan(self, name_or_id, ignore_missing=False):
        """Find a firewall plan

        :param id_or_name: The name or ID of a firewall plan.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.compute.v2.firewall_plan.FirewallPlan` or None
        """
        return self._find(_firewall_plan.FirewallPlan, name_or_id,
                          ignore_missing=ignore_missing)

    def reboot_firewall(self, firewall_id, type):
        """
        Reboot Firewall.

        :param firewall_id: ID of specified firewall.
        :param type: Type to reboot firewall.
        :return: ``None``
        """
        action = _firewall_action.FirewallAction()
        return action.reboot(self.session, firewall_id, type)

    def reset_password_firewall(self, firewall_id, username):
        """
        Reset password of firewall instance.

        :param firewall_id: ID of specified firewall.
        :param username: Username to change the password.
        :return: :class:`~ecl.network.v2.firewall_action.FirewallAction`
        """
        action = _firewall_action.FirewallAction()
        return action.reset_password(self.session, firewall_id, username)

    def load_balancers(self, **query):
        """
        List all visible load_balancers.

        :param query: Query parameters to select results.
        :return: A generator of LoadBalancer instances.
        :rtype: :class:`~ecl.network.v2.load_balancer.LoadBalancer`
        """
        return list(self._list(_load_balancer.LoadBalancer, 
                               paginated=False, **query))

    def create_load_balancer(self, load_balancer_plan_id,
                             availability_zone=None, description=None,
                             name=None):
        """
        Create a load_balancer.

        :param load_balancer_plan_id: Load Balancer Plan.
        :param availability_zone: Availability Zone.
        :param description: Description of the Load Balancer.
        :param name: Name of the Load Balancer.
        :return: :class:`~ecl.network.v2.load_balancer.LoadBalancer`
        """
        body = {"load_balancer_plan_id": load_balancer_plan_id}
        if availability_zone:
            body["availability_zone"] = availability_zone
        if description:
            body["description"] = description
        if name:
            body["name"] = name
        return self._create(_load_balancer.LoadBalancer, **body)

    def get_load_balancer(self, load_balancer_id):
        """
        Get details for load_balancer.

        :param load_balancer_id: ID of load balancer.
        :return: :class:`~ecl.network.v2.load_balancer.LoadBalancer`
        """
        return self._get(_load_balancer.LoadBalancer, load_balancer_id)

    def delete_load_balancer(self, load_balancer_id):
        """
        Delete load_balancer.

        :param load_balancer_id: ID of load balancer.
        :return: :class:`~ecl.network.v2.load_balancer.LoadBalancer`
        """
        return self._delete(_load_balancer.LoadBalancer, load_balancer_id)

    def find_load_balancer(self, name_or_id, ignore_missing=False):
        """Find a single load_balancer

        :param name_or_id: The name or ID of a load_balancer.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.load_balancer.LoadBalancer` or None
        """
        return self._find(_load_balancer.LoadBalancer,
                          name_or_id, ignore_missing=ignore_missing)

    def update_load_balancer(self, load_balancer_id, **params):
        """
        Update a specified firewall.

        :param load_balancer_id: ID of specified load balancer.
        :param \*\*params: Parameters for firewall update.

            * load_balancer_interface_id: ID for specified load_balancer_interface.
            * description: Description the Load Balancer Interface.
            * ip_address: IP Address.
            * name: Name the Load Balancer Interface.
            * network_id: Network ID.
            * virtual_ip_address: Virtual IP Address.
            * virtual_ip_properties: Properties used for virtual IP address.
                                     Available parameters include:
                * protocol: Redundancy Protocol. Enumeration is [‘vrrp’].
                            This parameter is optional.
                * vrid: VRRP group identifier. This parameter is optional.

        :return: :class:`~ecl.network.v2.load_balancer.LoadBalancer`
        """
        load_balancer = _load_balancer.LoadBalancer()
        return load_balancer.update(self.session, load_balancer_id, **params)

    def load_balancer_interfaces(self, **query):
        """
        List all visible load_balancer_interfaces.

        :param query: Query parameters to select results
        :return: A generator of LoadBalancerInterface instances.
        :rtype: :class:`~ecl.network.v2.load_balancer_interface.LoadBalancerInterface`
        """
        return list(self._list(_load_balancer_if.LoadBalancerInterface, **query))

    def get_load_balancer_interface(self, load_balancer_interface_id):
        """
        Get details for load_balancer_interface.

        :param load_balancer_interface_id: ID for specified load_balancer_interface.
        :return: :class:`~ecl.network.v2.load_balancer_interface.LoadBalancerInterface`
        """
        return self._get(_load_balancer_if.LoadBalancerInterface, load_balancer_interface_id)

    def update_load_balancer_interface(self, load_balancer_id,
                                       load_balancer_interface_id, **params):
        """
        Update load_balancer_interface.

        :param load_balancer_id: ID for parent load balancer of
                                 specified load_balancer_interface.
        :param load_balancer_interface_id: ID for specified
                                           load_balancer_interface.
        :param params: Parameters for update.
        :return:
        """
        interface = _load_balancer_if.LoadBalancerInterface()
        return interface.update(self.session, load_balancer_id,
                                load_balancer_interface_id, **params)

    def load_balancer_plans(self, **query):
        """
        List all visible load_balancer_plans.

        :param query: Query parameters to select results.
        :return: A generator of LoadBalancerPlan instances.
        :rtype: :class:`~ecl.network.v2.load_balancer_plan.LoadBalancerPlan`
        """
        return list(self._list(_load_balancer_plan.LoadBalancerPlan, **query))

    def get_load_balancer_plan(self, load_balancer_interface_id):
        """
        Get details for load_balancer_plan.

        :param load_balancer_interface_id: ID of specified load balancer plan.
        :return: :class:`~ecl.network.v2.load_balancer_plan.LoadBalancerPlan`
        """
        return self._get(_load_balancer_plan.LoadBalancerPlan, load_balancer_interface_id)

    def find_load_balancer_plan(self, name_or_id, ignore_missing=False):

        """Find a load_balancer plan

        :param id_or_name: The name or ID of a load balancer plan.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.compute.v2._load_balancer_plan.LoadBalancerPlan` or None
        """
        return self._find(_load_balancer_plan.LoadBalancerPlan, name_or_id,
                          ignore_missing=ignore_missing)

    def reboot_load_balancer(self, load_balancer_id, type):
        """
        Reboot Firewall.

        :param load_balancer_id: ID of specified load_balancer.
        :param type: Type to reboot firewall.
        :return: ``None``
        """
        action = _load_balancer_action.LoadBalancerAction()
        return action.reboot(self.session, load_balancer_id, type)

    def reset_password_load_balancer(self, load_balancer_id, username):
        """
        Reset password of load balancer instance.

        :param load_balancer_id: ID of specified load balancer.
        :param username: Username to change the password.
        :return: :class:`~ecl.network.v2.load_balancer_action.LoadBalancerAction`
        """
        action = _load_balancer_action.LoadBalancerAction()
        return action.reset_password(self.session, load_balancer_id, username)

    def load_balancer_syslog_servers(self, **query):
        """Return a list of Load Balancer Syslog Server

        :returns: A list of Load Balancer Syslog Server objects
        """
        return list(self._list(_load_balancer_syslog.LoadBalancerSyslogServer,
                               paginated=False, **query))

    def get_load_balancer_syslog_server(self, load_balancer_syslog_server):

        """Get a single Load Balancer Syslog Server

        :param load_balancer_syslog_server: The value can be the ID of
                        an Load Balancer Syslog Server or a
                        :class:`~ecl.network.v2.load_balancer_syslog_server.
                        LoadBalancerSyslogServer` instance.
        :returns: One :class:`~ecl.network.v2.load_balancer_syslog_server.
                  LoadBalancerSyslogServer`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_load_balancer_syslog.LoadBalancerSyslogServer,
                         load_balancer_syslog_server)

    def create_load_balancer_syslog_server(self, ip_address, load_balancer_id, name,
                                           acl_logging=None, appflow_logging=None,
                                           date_format=None, description=None,
                                           log_facility=None, log_level=None,
                                           port_number=None, priority=None,
                                           tcp_logging=None, tenant_id=None,
                                           time_zone=None, transport_type=None,
                                           user_configurable_log_messages=None):
        """Create a Load Balancer Syslog Server from properties

        :param string ip_address: IP address of the syslog server
        :param uuid load_balancer_id: ID of load balancer this syslog server belongs to
        :param string name: Name for syslog
        :param enumerate acl_logging: ["ENABLED", "DISABLED"]
        :param enumerate appflow_logging: ["ENABLED", "DISABLED"]
        :param enumerate date_format: ["DDMMYYYY", "MMDDYYYY", "YYYYMMDD"]
        :param string description: Description for syslog
        :param enumerate log_facility: ["LOCAL0", "LOCAL1", "LOCAL2",
                                        "LOCAL3", "LOCAL4", "LOCAL5",
                                        "LOCAL6", "LOCAL7"]
        :param string log_level: "ALL", "NONE", or one or more (join with "|") of
                                        ["ALERT", "CRITICAL", "NOTICE",
                                        "DEBUG", "ERROR", "WARNING", "EMERGENCY",
                                        "INFORMATIONAL"]
        :param int port_number: 1-65536
        :param int priority: 0-255
        :param enumerate tcp_logging: ["NONE", "ALL"]
        :param uuid tenant_id: Tenant ID of the owner
        :param enumerate time_zone: ["GMT_TIME", "LOCAL_TIME"]
        :param enumerate transport_type: ["UDP"]
        :param enumerate user_configurable_log_messages: ["YES", "NO"]

        :return: :class:`~ecl.network.v2.load_balancer_syslog_server.LoadBalancerSyslogServer`
        """
        body = {
            "ip_address": ip_address,
            "load_balancer_id": load_balancer_id,
            "name": name
        }
        if acl_logging is not None:
            body["acl_logging"] = acl_logging
        if appflow_logging is not None:
            body["appflow_logging"] = appflow_logging
        if date_format is not None:
            body["date_format"] = date_format
        if description is not None:
            body["description"] = description
        if log_facility is not None:
            body["log_facility"] = log_facility
        if log_level is not None:
            body["log_level"] = log_level
        if port_number is not None:
            body["port_number"] = port_number
        if priority is not None:
            body["priority"] = priority
        if tcp_logging is not None:
            body["tcp_logging"] = tcp_logging
        if tenant_id is not None:
            body["tenant_id"] = tenant_id
        if time_zone is not None:
            body["time_zone"] = time_zone
        if transport_type is not None:
            body["transport_type"] = transport_type
        if user_configurable_log_messages is not None:
            body["user_configurable_log_messages"] = user_configurable_log_messages
        return self._create(_load_balancer_syslog.LoadBalancerSyslogServer, **body)

    def update_load_balancer_syslog_server(self, load_balancer_syslog_server,
                                           acl_logging=None, appflow_logging=None,
                                           date_format=None, description=None,
                                           log_facility=None, log_level=None,
                                           priority=None, tcp_logging=None,
                                           time_zone=None,
                                           user_configurable_log_messages=None):
        """Update a Load Balancer Syslog Server from properties

        :param load_balancer_syslog_server: The value can be
                            either the ID of an Load Balancer Syslog Server or a
                                   :class:`~ecl.network.v2.load_balancer_syslog_server.
                                   LoadBalancerSyslogServer` instance.
        :param enumerate acl_logging: ["ENABLED", "DISABLED"]
        :param enumerate appflow_logging: ["ENABLED", "DISABLED"]
        :param enumerate date_format: ["DDMMYYYY", "MMDDYYYY", "YYYYMMDD"]
        :param string description: Description for syslog
        :param enumerate log_facility: ["LOCAL0", "LOCAL1", "LOCAL2",
                                        "LOCAL3", "LOCAL4", "LOCAL5",
                                        "LOCAL6", "LOCAL7"]
        :param string log_level: "ALL", "NONE", or one or more (join with "|") of
                                        ["ALERT", "CRITICAL", "NOTICE",
                                        "DEBUG", "ERROR", "WARNING", "EMERGENCY",
                                        "INFORMATIONAL"]
        :param int priority: 0-255
        :param enumerate tcp_logging: ["NONE", "ALL"]
        :param enumerate time_zone: ["GMT_TIME", "LOCAL_TIME"]
        :param enumerate user_configurable_log_messages: ["YES", "NO"]

        :return: :class:`~ecl.network.v2.load_balancer_syslog_server.LoadBalancerSyslogServer`
        """
        body = dict()
        if acl_logging is not None:
            body["acl_logging"] = acl_logging
        if appflow_logging is not None:
            body["appflow_logging"] = appflow_logging
        if date_format is not None:
            body["date_format"] = date_format
        if description is not None:
            body["description"] = description
        if log_facility is not None:
            body["log_facility"] = log_facility
        if log_level is not None:
            body["log_level"] = log_level
        if priority is not None:
            body["priority"] = priority
        if tcp_logging is not None:
            body["tcp_logging"] = tcp_logging
        if time_zone is not None:
            body["time_zone"] = time_zone
        if user_configurable_log_messages is not None:
            body["user_configurable_log_messages"] = user_configurable_log_messages
        if not isinstance(load_balancer_syslog_server,
                          _load_balancer_syslog.LoadBalancerSyslogServer):
            load_balancer_syslog_server = self._get_resource(
                        _load_balancer_syslog.LoadBalancerSyslogServer,
                        load_balancer_syslog_server
            )
            load_balancer_syslog_server._body.clean()
        return self._update(_load_balancer_syslog.LoadBalancerSyslogServer,
                            load_balancer_syslog_server, **body)

    def delete_load_balancer_syslog_server(self, load_balancer_syslog_server,
                                           ignore_missing=False):
        """Delete a load balancer syslog server

        :param load_balancer_syslog_server: The value can be
                either the ID of an Load Balancer Syslog Server or a
                       :class:`~ecl.network.v2.load_balancer_syslog_server.
                       LoadBalancerSyslogServer` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the Load Balancer Syslog Server does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent Load Balancer Syslog Server.
        :returns: ``None``
        """
        return self._delete(_load_balancer_syslog.LoadBalancerSyslogServer,
                            load_balancer_syslog_server,
                            ignore_missing=ignore_missing)

    def internet_services(self, **query):
        """Return a list of Internet Services

        :returns: A list of Internet Service objects
        """
        return list(self._list(_internet.InternetService, paginated=False, **query))

    def get_internet_service(self, internet_service):

        """Get a single Internet Service

        :param internet_service: The value can be the ID of an Internet Service or a
                       :class:`~ecl.network.v2.internet.InternetService` instance.

        :returns: One :class:`~ecl.network.v2.internet.InternetService`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_internet.InternetService, internet_service)

    def find_internet_service(self,name_or_id, ignore_missing=False):
        """Find a single internet_service

        :param name_or_id: The name or ID of an internet_service.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.internet.InternetService` or None
        """
        return self._find(_internet.InternetService,
                          name_or_id, ignore_missing=ignore_missing)

    def internet_gateways(self, **query):
        """Return a list of Internet Gateways

        :param query: Query parameters to select results

        :returns: A list of Internet Gateway objects
        """
        return list(self._list(_internet.InternetGateway, paginated=False, **query))

    def get_internet_gateway(self, internet_gateway):

        """Get a single Internet Gateway

        :param internet_gateway: The value can be the ID of an Internet Gateway or a
                       :class:`~ecl.network.v2.internet.InternetGateway` instance.

        :returns: One :class:`~ecl.network.v2.internet.InternetGateway`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_internet.InternetGateway, internet_gateway)

    def create_internet_gateway(self, internet_service_id, qos_option_id,
                                name=None, description=None, tenant_id=None):
        """Create a new Internet Gateway from parameters.

        :param string internet_service_id: ID of Internet Service of gateway to create
        :param string qos_option_id: ID of QoS Option of gateway to create
        :param string name: name of gateway to create
        :param string description: description of gateway to create
        :param string tenant_id: ID of Tenant of gateway to create

        :return: One :class:`~ecl.network.v2.internet.InternetGateway`
        """
        body = {
            "internet_service_id": internet_service_id,
            "qos_option_id": qos_option_id
        }
        if name is not None:
            body.update({"name": name})
        if description is not None:
            body.update({"description": description})
        if tenant_id is not None:
            body.update({"tenant_id": tenant_id})

        return self._create(_internet.InternetGateway, **body)

    def update_internet_gateway(self, inet_gw, **body):
        """Update a new Internet Gateway from parameters.

        :param inet_gw: The value can be the ID of an Internet Gateway or a
                       :class:`~ecl.network.v2.internet.InternetGateway` instance.
        :param string name: name of gateway to update
        :param string description: description of gateway to update
        :param string qos_option_id: ID of QoS Option of gateway to update

        :return: result of update Internet Gateway
        :rtype :class:`~ecl.network.v2.internet.InternetGateway`

        """
        if not isinstance(inet_gw, _internet.InternetGateway):
            inet_gw = self._get_resource(_internet.InternetGateway, inet_gw)
            inet_gw._body.clean()
        return self._update(_internet.InternetGateway, inet_gw, **body)

    def delete_internet_gateway(self, inet_gw, ignore_missing=False):
        """Delete an Internet Gateway

        :param inet_gw: The value can be either the ID of an Internet Gateway or a
                       :class:`~ecl.network.v2.internet.InternetGateway` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the Internet Gateway does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent Internet Gateway.

        :returns: ``None``
        """
        return self._delete(_internet.InternetGateway, inet_gw,
                            ignore_missing=ignore_missing)

    def find_internet_gateway(self, name_or_id, ignore_missing=False):
        """Find a single internet_gateway

        :param name_or_id: The name or ID of an internet_gateway.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.internet.InternetGateway` or None
        """
        return self._find(_internet.InternetGateway,
                          name_or_id, ignore_missing=ignore_missing)

    def qos_options(self, **query):
        """Return a list of QoS Options

        :returns: A list of QoS Option objects
        """
        return list(self._list(_qos_option.QosOption, paginated=False,
                               **query))

    def get_qos_option(self, qos_option):
        """Get a single QoS Option

        :param qos_option: The value can be the ID of a QoS Option or a
                       :class:`~ecl.network.v2.qos_option.QoSOption` instance.

        :returns: One :class:`~ecl.network.v2.qos_option.QoSOption`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_qos_option.QosOption, qos_option)

    def find_qos_option(self, name_or_id, ignore_missing=False):
        """Find a single qos_option

        :param name_or_id: The name or ID of a qos_option.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.qos_option.QosOption` or None
        """
        return self._find(_qos_option.QosOption,
                          name_or_id, ignore_missing=ignore_missing)

    def public_ips(self, **query):
        """Return a list of Public IPs

        :param query: Query parameters to select results

        :returns: A list of Public IP objects
        """
        return list(self._list(_publicip.PublicIP, paginated=False, **query))

    def get_public_ip(self, publicip):
        """Get a single Public IP

        :param publicip: The value can be the ID of a Public IP or a
                       :class:`~ecl.network.v2.publicip.PublicIP` instance.

        :returns: One :class:`~ecl.network.v2.publicip.PublicIP`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_publicip.PublicIP, publicip)

    def create_public_ip(self, internet_gw_id, submask_length,
                         description=None, name=None, tenant_id=None):
        """Create a Public IP from parameters

        :param string internet_gw_id: ID of Internet Gateway to create public ip
        :param int submask_length: submask length to create public ip pool with
        :param string description: description of public ip to create
        :param string name: name of public ip to create
        :param string tenant_id: tenant ID of public ip

        :return: One :class:`~ecl.network.v2.publicip.PublicIP`
        """

        body = {"internet_gw_id": internet_gw_id,
                "submask_length": submask_length}
        if description:
            body.update({"description": description})
        if name:
            body.update({"name": name})
        if tenant_id:
            body.update({"tenant_id": tenant_id})
        return self._create(_publicip.PublicIP, **body)

    def update_public_ip(self, public_ip, **body):
        """Update a Public IP

        :param public_ip:  The value can be the ID of a Public IP or a
                       :class:`~ecl.network.v2.publicip.PublicIP` instance.
        :param description: description of public ip to update

        :returns: The result of update Public IP
        :rtype: :class:`~ecl.network.v2.publicip.PublicIP`
        """
        if not isinstance(public_ip, _publicip.PublicIP):
            public_ip = self._get_resource(_publicip.PublicIP, public_ip)
            public_ip._body.clean()
        return self._update(_publicip.PublicIP, public_ip, **body)

    def delete_public_ip(self, publicip, ignore_missing=False):
        """Delete a Public Ip

        :param publicip: The value can be either the ID of a Public IP or a
                       :class:`~ecl.network.v2.publicip.PublicIP` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the Public IP does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent Public IP.

        :return: One :class:`~ecl.network.v2.publicip.PublicIP`
        """
        return self._delete(_publicip.PublicIP, publicip,
                            ignore_missing=ignore_missing)

    def gateway_interfaces(self, **query):
        """Return a list of Gateway Interfaces

        :returns: A list of Gateway Interface objects
        """
        return list(self._list(_gwif.GatewayInterface, paginated=False,
                               **query))

    def get_gateway_interface(self, gw_interface):
        """Get a single Gateway Interface

        :param gw_interface: The value can be the ID of a Gateway Interface or a
                       :class:`~ecl.network.v2.gw_interface.GatewayInterface` instance.

        :returns: One :class:`~ecl.network.v2.gw_interface.GatewayInterface`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_gwif.GatewayInterface, gw_interface)

    def create_gateway_interface(self, service_type, network_id, netmask,
                                 primary_ipv4, secondary_ipv4, gw_vipv4, vrid,
                                 name=None, description=None,
                                 primary_ipv6=None, secondary_ipv6=None,
                                 gw_vipv6=None, interdc_gw_id=None,
                                 internet_gw_id=None,
                                 vpn_gw_id=None, aws_gw_id=None, gcp_gw_id=None,
                                 azure_gw_id=None, tenant_id=None):
        """Create a Gateway Interface from parameters

        :param string service_type: service type of gateway interface to create
        :param string network_id: network ID of gateway interface to create
        :param int netmask: netmask of gateway interface to create
        :param IPv4 primary_ipv4: primary IPv4 of gateway interface to create
        :param IPv4 secondary_ipv4: secondary IPv4 of gateway interface to create
        :param IPv4 gw_vipv4: gateway virtual IPv4 interface to create
        :param int vrid: ID of VRRP of gateway interface to create
        :param string name: name of gateway interface to create
        :param string description: description of gateway interface to create
        :param IPv6 primary_ipv6: primary IPv6 of gateway interface to create
        :param IPv6 secondary_ipv6: secondary IPv6 of gateway interface to create
        :param IPv6 gw_vipv6: gateway virtual IPv6 of gateway interface to create
        :param string interdc_gw_id: InterDC Gateway ID of gateway interface to create
        :param string internet_gw_id: Internet Gateway ID of gateway interface to create
        :param string vpn_gw_id: VPN Gateway ID of gateway interface to create
        :param string tenant_id: tenant ID of gateway interface to create

        :returns: The results of GatewayInterface creation
        :rtype: One :class:`~ecl.network.v2.gwif.GatewayInterface`
        """
        body = {
            "service_type": service_type,
            "network_id": network_id,
            "netmask": netmask,
            "primary_ipv4": primary_ipv4,
            "secondary_ipv4": secondary_ipv4,
            "gw_vipv4": gw_vipv4,
            "vrid": vrid
        }
        if name:
            body.update({"name": name})
        if description:
            body.update({"description": description})
        if primary_ipv6:
            body.update({"primary_ipv6": primary_ipv6})
        if secondary_ipv6:
            body.update({"secondary_ipv6": secondary_ipv6})
        if gw_vipv6:
            body.update({"gw_vipv6": gw_vipv6})
        if interdc_gw_id:
            body.update({"interdc_gw_id": interdc_gw_id})
        if internet_gw_id:
            body.update({"internet_gw_id": internet_gw_id})
        if vpn_gw_id:
            body.update({"vpn_gw_id": vpn_gw_id})
        if aws_gw_id:
            body.update({"aws_gw_id": aws_gw_id})
        if gcp_gw_id:
            body.update({"gcp_gw_id": gcp_gw_id})
        if azure_gw_id:
            body.update({"azure_gw_id": azure_gw_id})
        if tenant_id:
            body.update({"tenant_id": tenant_id})

        return self._create(_gwif.GatewayInterface, **body)

    def update_gateway_interface(self, gw_interface, **body):
        """Update a Gateway Interface

        :param gw_interface:  The value can be the ID of a Gateway Interface or a
                       :class:`~ecl.network.v2.gw_interface.GatewayInterface` instance.
        :param name: name of gateway interface to update
        :param description: description of gateway interface to update

        :returns: The result of update Gateway Interface
        :rtype: :class:`~ecl.network.v2.gw_interface.GatewayInterface`
        """
        if not isinstance(gw_interface, _gwif.GatewayInterface):
            gw_interface = self._get_resource(_gwif.GatewayInterface, gw_interface)
            gw_interface._body.clean()
        return self._update(_gwif.GatewayInterface, gw_interface, **body)

    def delete_gateway_interface(self, gwif, ignore_missing=False):
        """Delete a Gateway Interface

        :param gwif: The value can be either the ID of a Gateway Interface or a
                       :class:`~ecl.network.v2.gwif.GatewayInterface` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the Gateway Interface does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent Gateway Interface.

        :return: One :class:`~ecl.network.v2.gwif.GatewayInterface`
        """
        return self._delete(_gwif.GatewayInterface, gwif,
                            ignore_missing=ignore_missing)

    def find_gateway_interface(self, name_or_id, ignore_missing=False):
        """Find a single gateway_interface

        :param name_or_id: The name or ID of a common_function_gatway.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.gwif.GatewayInterface` or None
        """
        return self._find(_gwif.GatewayInterface,
                          name_or_id, ignore_missing=ignore_missing)

    def static_routes(self, **query):
        """Return a list of Static Routes

        :returns: A list of Static Route objects
        """
        return list(self._list(_static_route.StaticRoute, paginated=False,
                               **query))

    def get_static_route(self, static_route):
        """Get a single Static Route

        :param static_route: The value can be the ID of a  Static Route or a
                       :class:`~ecl.network.v2.static_route.StaticRoute` instance.

        :returns: One :class:`~ecl.network.v2.static_route.StaticRoute`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_static_route.StaticRoute, static_route)

    def create_static_route(self, service_type, destination, nexthop,
                            name=None, description=None,interdc_gw_id=None,
                            internet_gw_id=None, vpn_gw_id=None, aws_gw_id=None,
                            gcp_gw_id=None, azure_gw_id=None, tenant_id=None):
        """Create a Static Route

        :param service_type: service type of static route to create
        :param destination: destination of static route to create
        :param nexthop: next hop of static route to create
        :param name: name of static route to create
        :param description: description of static route to create
        :param interdc_gw_id: InterDC Gateway ID of static route to create
        :param internet_gw_id: Internet Gateway ID of static route to create
        :param vpn_gw_id: VPN Gateway ID of static route to create
        :param tenant_id: tenant ID of static route to create

        :return: The results of StaticRoute creation
        :rtype:  One :class:`~ecl.network.v2.staticroute.StaticRoute`
        """
        body = {
            "service_type": service_type,
            "destination": destination,
            "nexthop": nexthop
        }
        if name:
            body.update({"name": name})
        if description:
            body.update({"description": description})
        if interdc_gw_id:
            body.update({"interdc_gw_id": interdc_gw_id})
        if internet_gw_id:
            body.update({"internet_gw_id": internet_gw_id})
        if vpn_gw_id:
            body.update({"vpn_gw_id": vpn_gw_id})
        if aws_gw_id:
            body.update({"aws_gw_id": aws_gw_id})
        if gcp_gw_id:
            body.update({"gcp_gw_id": gcp_gw_id})
        if azure_gw_id:
            body.update({"azure_gw_id": azure_gw_id})
        if tenant_id:
            body.update({"tenant_id": tenant_id})

        return self._create(_static_route.StaticRoute, **body)

    def update_static_route(self, static_route, **body):
        """Update a Static Route

        :param static_route:  The value can be the ID of a Static Route or a
                       :class:`~ecl.network.v2.static_route.StaticRoute` instance.
        :param name: name of static route to update
        :param description: description of static route to update

        :returns: The result of update Static Route
        :rtype: :class:`~ecl.network.v2.static_route.StaticRoute`
        """
        if not isinstance(static_route, _static_route.StaticRoute):
            static_route = self._get_resource(_static_route.StaticRoute, static_route)
            static_route._body.clean()
        return self._update(_static_route.StaticRoute, static_route, **body)

    def delete_static_route(self, static_route, ignore_missing=False):
        """Delete a Static Route

        :param static_route: The value can be either the ID of a Static Route or a
                       :class:`~ecl.network.v2.static_route.StaticRoute` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the Static Route does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent Static Route.

        :return: One :class:`~ecl.network.v2.static_route.StaticRoute`
        """
        return self._delete(_static_route.StaticRoute, static_route,
                            ignore_missing=ignore_missing)

    def vpn_services(self, **query):
        """Return a list of VPN Service

        :returns: A list of VPN Service objects
        """
        return list(self._list(_vpn.VPNService, paginated=False,
                               **query))

    def get_vpn_service(self, vpn_service):

        """Get a single VPN Service

        :param vpn_service: The value can be the ID of a VPN Service or a
                       :class:`~ecl.network.v2.vpn.VPNService` instance.

        :returns: One :class:`~ecl.network.v2.vpn.VPNService`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_vpn.VPNService, vpn_service)

    def vpn_gateways(self, **query):
        """Return a list of VPN Gateways

        :param query: Query parameters to select results

        :returns: A list of VPN Gateway objects
        """
        return list(self._list(_vpn.VPNGateway, paginated=False, **query))

    def get_vpn_gateway(self, vpn_gateway):

        """Get a single VPN Gateway

        :param vpn_gateway: The value can be the ID of a VPN Gateway or a
                       :class:`~ecl.network.v2.vpn.VPNGateway` instance.

        :returns: One :class:`~ecl.network.v2.vpn.VPNGateway`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_vpn.VPNGateway, vpn_gateway)

    def find_vpn_gateway(self, name_or_id, ignore_missing=False):
        """Find a single vpn_gateway

        :param name_or_id: The name or ID of a vpn_gateway.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.vpn.VPNGateway` or None
        """
        return self._find(_vpn.VPNGateway,
                          name_or_id, ignore_missing=ignore_missing)

    def vpn_interfaces(self, **query):
        """Return a list of VPN Interface

        :returns: A list of VPN Interface objects
        """
        return list(self._list(_vpn.VPNInterface, paginated=False,
                               *query))

    def get_vpn_interface(self, vpn_interface):

        """Get a single VPN Gateway

        :param vpn_interface: The value can be the ID of a VPN Interface or a
                       :class:`~ecl.network.v2.vpn.VPNInterface` instance.

        :returns: One :class:`~ecl.network.v2.vpn.VPNInterface`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_vpn.VPNInterface, vpn_interface)

    def interdc_services(self, **query):
        """Return a list of InterDC Service

        :returns: A list of InterDC Service objects
        """
        return list(self._list(_interdc.InterDCService, paginated=False,
                               **query))

    def get_interdc_service(self, interdc_service):

        """Get a single InterDC Service

        :param interdc_service: The value can be the ID of an InterDC Service or a
                       :class:`~ecl.network.v2.vpn.InterDCService` instance.

        :returns: One :class:`~ecl.network.v2.interdc.InterDCService`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_interdc.InterDCService, interdc_service)

    def interdc_gateways(self, **query):
        """Return a list of InterDC Gateways

        :param query: Query parameters to select results

        :returns: A list of InterDC Gateway objects
        """
        return list(self._list(_interdc.InterDCGateway, paginated=False, **query))

    def get_interdc_gateway(self, interdc_gateway):

        """Get a single InterDC Gateway

        :param interdc_gateway: The value can be the ID of an InterDC Gateway or a
                       :class:`~ecl.network.v2.interdc.InterDCGateway` instance.

        :returns: One :class:`~ecl.network.v2.interdc.InterDCGateway`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_interdc.InterDCGateway, interdc_gateway)

    def find_interdc_gateway(self, name_or_id, ignore_missing=False):
        """Find a single interdc_gateway

        :param name_or_id: The name or ID of an interdc_gateway.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.interdc.InternetDC` or None
        """
        return self._find(_interdc.InterDCGateway,
                          name_or_id, ignore_missing=ignore_missing)

    def interdc_interfaces(self, **query):
        """Return a list of InterDC Interface

        :returns: A list of InterDC Interface objects
        """
        return list(self._list(_interdc.InterDCInterface, paginated=False,
                               **query))

    def get_interdc_interface(self, interdc_interface):

        """Get a single InterDC Gateway

        :param interdc_interface: The value can be the ID of an InterDC Interface or a
                       :class:`~ecl.network.v2.interdc.InterDCInterface` instance.

        :returns: One :class:`~ecl.network.v2.interdc.InterDCInterface`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_interdc.InterDCInterface, interdc_interface)

    def create_interdc_interface(self,  interdc_gw_id, netmask,
                                 primary_ipv4, secondary_ipv4, gw_vipv4, vrid,
                                 name=None, description=None,
                                 primary_ipv6=None, secondary_ipv6=None,
                                 gw_vipv6=None, tenant_id=None):
        """Create an InterDC Interface from parameters

        :param string interdc_gw_id: ID of InterDC interface to create
        :param int netmask: netmask of InterDC interface to create
        :param IPv4 primary_ipv4: primary IPv4 of InterDC interface to create
        :param IPv4 secondary_ipv4: secondary IPv4 of InterDC interface to create
        :param IPv4 gw_vipv4: InterDC virtual IPv4 interface to create
        :param int vrid: ID of VRRP of InterDC interface to create
        :param string name: name of InterDC interface to create
        :param string description: description of InterDC interface to create
        :param IPv6 primary_ipv6: primary IPv6 of InterDC interface to create
        :param IPv6 secondary_ipv6: secondary IPv6 of InterDC interface to create
        :param IPv6 gw_vipv6: InterDC virtual IPv6 of InterDC interface to create
        :param string tenant_id: tenant ID of InterDC interface to create

        :returns: The results of InterDCInterface creation
        :rtype: One :class:`~ecl.network.v2.interdc.InterDCInterface`
        """
        body = {
            "interdc_gw_id": interdc_gw_id,
            "netmask": netmask,
            "primary_ipv4": primary_ipv4,
            "secondary_ipv4": secondary_ipv4,
            "gw_vipv4": gw_vipv4,
            "vrid": vrid
        }
        if name:
            body.update({"name": name})
        if description:
            body.update({"description": description})
        if primary_ipv6:
            body.update({"primary_ipv6": primary_ipv6})
        if secondary_ipv6:
            body.update({"secondary_ipv6": secondary_ipv6})
        if gw_vipv6:
            body.update({"gw_vipv6": gw_vipv6})
        if tenant_id:
            body.update({"tenant_id": tenant_id})

        return self._create(_interdc.InterDCInterface, **body)

    def update_interdc_interface(self, interdc_interface, **body):
        """Update an InterDC Interface from parameters

        :param interdc_interface: ID of InterDC interface to update
        :param name: name of InterDC interface to update
        :param description: description of InterDC interface to update

        :returns: The results of InterDCInterface update
        :rtype: One :class:`~ecl.network.v2.interdc.InterDCInterface`
        """
        if not isinstance(interdc_interface, _interdc.InterDCInterface):
            interdc_interface = self._get_resource(_interdc.InterDCInterface, interdc_interface)
            interdc_interface._body.clean()

        return self._update(_interdc.InterDCInterface, interdc_interface, **body)

    def delete_interdc_interface(self, interdc_interface, ignore_missing=False):
        """Delete an InterDC Interface

        :param gwif: The value can be either the ID of an InterDC Interface or a
                       :class:`~ecl.network.v2.gwif.InterDCInterface` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the InterDC Interface does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent InterDC Interface.

        :return: One :class:`~ecl.network.v2.interdc.InterDCInterface`
        """
        return self._delete(_interdc.InterDCInterface, interdc_interface,
                            ignore_missing=ignore_missing)

    def common_function_pools(self, **query):
        """Return a list of common_function_pools

        :returns: A list of common_function_pool objects
        :rtype: list of :class:`~ecl.network.v2.common_function_pool.CommonFunctionPool`
        """
        return list(self._list(_common_function_pool.CommonFunctionPool,
                               paginated=False, **query))

    def get_common_function_pool(self, common_function_pool):
        """Get a single common_function_pool

        :param common_function_pool:
            The value can be the ID of a common_function_pool or a
            :class:`~ecl.network.v2.common_function_pool.CommonFunctionPool` instance.

        :returns: One :class:`~ecl.network.v2.common_function_pool.CommonFunctionPool`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_common_function_pool.CommonFunctionPool,
                         common_function_pool)

    def find_common_function_pool(self, name_or_id, ignore_missing=False):
        """Find a single common_function_pool

        :param name_or_id: The name or ID of a common_function_pool.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.common_function_pool.CommonFunctionPool` or None
        """
        return self._find(_common_function_pool.CommonFunctionPool,
                          name_or_id, ignore_missing=ignore_missing)

    def common_functions(self, **query):
        """Return a list of common_functions

        :param query: Query parameters to select results.
        :returns: A list of common_functions objects
        :rtype: list of :class:`~ecl.network.v2.common_function.CommonFunction`
        """
        return list(self._list(_common_function.CommonFunction,
                               paginated=False, **query))

    def get_common_function(self, common_function):
        """Get a single common_function

        :param common_function:
            The value can be the ID of a common_function or a
            :class:`~ecl.network.v2.common_function.CommonFunction` instance.

        :returns: One :class:`~ecl.network.v2.common_function.CommonFunction`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_common_function.CommonFunction,
                         common_function)

    def find_common_function(self, name_or_id, ignore_missing=False):
        """Find a single common_function

        :param name_or_id: The name or ID of a common_function_gatway.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.common_function.CommonFunction` or None
        """
        return self._find(_common_function.CommonFunction,
                          name_or_id, ignore_missing=ignore_missing)

    def common_function_gateways(self, **query):
        """Return a list of common_function_gateways

        :param query: Query parameters to select results.
        :returns: A list of common_function_gateways objects
        :rtype: list of :class:`~ecl.network.v2.common_function_gateway.CommonFunctionGateway`
        """
        return list(self._list(_common_function_gateway.CommonFunctionGateway,
                               paginated=False, **query))

    def get_common_function_gateway(self, common_function_gateway):
        """Get a single common_function_gateway

        :param common_function_gateway:
            The value can be the ID of a common_function_gateway or a
            :class:`~ecl.network.v2.common_function_gateway.CommonFunctionGateway` instance.

        :returns: One :class:`~ecl.network.v2.common_function_gateway.CommonFunctionGateway`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_common_function_gateway.CommonFunctionGateway,
                         common_function_gateway)

    def create_common_function_gateway(self, common_function_pool_id,
                                       description=None, name=None):
        """Create a new common_function_gateway from attributes

        :param string common_function_pool_id: Common Function Pool instantiated
            by this Gateway.
        :param string description: Description of the Common Function Pool
            Gateway resource.
        :param string name: Name of the Common Function Pool Gateway resource.

        :returns: The results of common_function_gateway creation
        :rtype: :class:`~ecl.network.v2.common_function_gateway.CommonFunctionGateway`
        """
        body = {"common_function_pool_id" :common_function_pool_id}
        if description:
            body["description"] = description
        if name:
            body["name"] = name
        return self._create(_common_function_gateway.CommonFunctionGateway,
                            **body)

    def update_common_function_gateway(self, common_function_gateway,
                                       **params):
        """Update a common_function_gateway

        :param common_function_gateway: The value can be the ID
            of a common_function_gateway or
            a :class:`~ecl.network.v2.common_function_gateway.CommonFunctionGateway` instance.

        :param kwargs \*\*params: Parameter for Common Function Gateway update.
            * description: Description of the Common Function Pool Gateway
                           resource.
            * name: Name of the Common Function Pool Gateway resource.

        :returns: The result of update common_function_gateway.
        :rtype: :class:`~ecl.network.v2.common_function_gateway.CommonFunctionGateway`
        """
        if not isinstance(common_function_gateway,
            _common_function_gateway.CommonFunctionGateway):
            common_function_gateway = \
                self._get_resource(_common_function_gateway.CommonFunctionGateway,
                    common_function_gateway)
            common_function_gateway._body.clean()
        return self._update(_common_function_gateway.CommonFunctionGateway,
                            common_function_gateway, **params)

    def delete_common_function_gateway(self, common_function_gateway,
                                       ignore_missing=False):
        """Delete a common_function_gateway

        :param common_function_gateway: The value can be either the ID
            of a common_function_gateway Interface or a
            :class:`~ecl.network.v2.common_function_gateway.CommonFunctionGateway` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the common_function_gateway Interface does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent common_function_gateway Interface.

        :return: One :class:`~ecl.network.v2.common_function_gateway.CommonFunctionGateway`
        """
        self._delete(_common_function_gateway.CommonFunctionGateway,
                     common_function_gateway, ignore_missing=ignore_missing)

    def find_common_function_gateway(self, name_or_id, ignore_missing=False):
        """Find a single common_function_gateway

        :param name_or_id: The name or ID of a common_function_gatway.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.common_function_gateway.CommonFunctionGateway` or None
        """
        return self._find(_common_function_gateway.CommonFunctionGateway,
                          name_or_id, ignore_missing=ignore_missing)

    def colocation_spaces(self, **query):
        """Return a list of colocation_spaces

        :param query: Query parameters to select results.
        :returns: A list of colocation_spaces objects
        :rtype: list of :class:`~ecl.network.v2.colocation_space.ColocationSpace`
        """
        return list(self._list(_colocation_space.ColocationSpace,
                               paginated=False, **query))

    def get_colocation_space(self, colocation_space):
        """Get a single colocation_space

        :param colocation_space:
            The value can be the ID of a colocation_space or a
            :class:`~ecl.network.v2.colocation_space.ColocationSpace` instance.

        :returns: One :class:`~ecl.network.v2.colocation_space.ColocationSpace`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_colocation_space.ColocationSpace,
                         colocation_space)

    def colocation_physical_links(self, **query):
        """Return a list of colocation_physical_links

        :param query: Query parameters to select results.
        :returns: A list of colocation_physical_links objects
        :rtype: list of :class:`~ecl.network.v2.colocation_physical_link.ColocationPhysicalLink`
        """
        return list(self._list(_colocation_physical_link.ColocationPhysicalLink,
                               paginated=False, **query))

    def get_colocation_physical_link(self, colocation_physical_link):
        """Get a single colocation_physical_link

        :param colocation_physical_link:
            The value can be the ID of a colocation_physical_link or a
            :class:`~ecl.network.v2.colocation_physical_link.ColocationPhysicalLink` instance.

        :returns: One :class:`~ecl.network.v2.colocation_physical_link.ColocationPhysicalLink`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_colocation_physical_link.ColocationPhysicalLink,
                         colocation_physical_link)

    def colocation_logical_links(self, **query):
        """Return a list of colocation_logical_links

        :param query: Query parameters to select results.
        :returns: A list of colocation_logical_links objects
        :rtype: list of :class:`~ecl.network.v2.colocation_logical_link.ColocationLogicalLink`
        """
        return list(self._list(_colocation_logical_link.ColocationLogicalLink,
                               paginated=False, **query))

    def get_colocation_logical_link(self, colocation_logical_link):
        """Get a single colocation_logical_link

        :param colocation_logical_link:
            The value can be the ID of a colocation_logical_link or a
            :class:`~ecl.network.v2.colocation_logical_link.ColocationLogicalLink` instance.

        :returns: One :class:`~ecl.network.v2.colocation_logical_link.ColocationLogicalLink`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_colocation_logical_link.ColocationLogicalLink,
                         colocation_logical_link)

    def create_colocation_logical_link(self, colocation_physical_link_id,
                                       network_id, vlan_id, description=None,
                                       name=None, tags=None):
        """Create a new colocation_logical_link from attributes

        :param string colocation_physical_link_id: Colocation Physical Link ID.
        :param string network_id: Network connected to this link.
        :param string vlan_id: Logical port vlan id.
        :param string description: Colocation Logical Link description.
        :param string name: Colocation Logical Link name.
        :param dict tags: Colocation Logical Link tags.

        :returns: The results of colocation_logical_link creation
        :rtype: :class:`~ecl.network.v2.colocation_logical_link.ColocationLogicalLink`
        """
        body = {
            "colocation_physical_link_id": colocation_physical_link_id,
            "network_id": network_id,
            "vlan_id": vlan_id
        }
        if description:
            body["description"] = description
        if name:
            body["name"] = name
        if tags:
            body["tags"] = tags
        return self._create(_colocation_logical_link.ColocationLogicalLink,
                            **body)

    def update_colocation_logical_link(self, colocation_logical_link,
                                       **params):
        """Update a colocation_logical_link

        :param colocation_logical_link: The value can be the ID
            of a colocation_logical_link or
            a :class:`~ecl.network.v2.colocation_logical_link.ColocationLogicalLink` instance.
        :param kwargs \*\*params: Parameters for Colocation Logical Link update.

            * string description: Colocation Logical Link description.
            * string name: Colocation Logical Link name.
            * dict tags: Colocation Logical Link tags.

        :returns: The result of update colocation_logical_link.
        :rtype: :class:`~ecl.network.v2.colocation_logical_link.ColocationLogicalLink`
        """
        if not isinstance(colocation_logical_link,
            _colocation_logical_link.ColocationLogicalLink):
            colocation_logical_link = \
                self._get_resource(_colocation_logical_link.ColocationLogicalLink,
                    colocation_logical_link)
            colocation_logical_link._body.clean()
        return self._update(_colocation_logical_link.ColocationLogicalLink,
                            colocation_logical_link, **params)

    def delete_colocation_logical_link(self, colocation_logical_link,
                                       ignore_missing=False):
        """Delete a colocation_logical_link

        :param colocation_logical_link: The value can be either the ID
            of a colocation_logical_link Interface or a
            :class:`~ecl.network.v2.colocation_logical_link.ColocationLogicalLink` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the colocation_logical_link Interface does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent colocation_logical_link Interface.

        :return: One :class:`~ecl.network.v2.colocation_logical_link.ColocationLogicalLink`
        """
        self._delete(_colocation_logical_link.ColocationLogicalLink,
                     colocation_logical_link, ignore_missing=ignore_missing)

    def aws_services(self, **params):
        """Return a list of aws_services

        :param params: The parameters as query string
                       to get aws_services by specified condition.
        :returns: A list of aws_services objects
        :rtype: list of :class:`~ecl.network.v2.aws.AWSService`
        """
        return list(self._list(_aws.AWSService, paginated=False, **params))

    def get_aws_service(self, aws_service):
        """Get a single aws_service

        :param aws_service:
            The value can be the ID of a aws_service or a
            :class:`~ecl.network.v2.aws.AWSService` instance.

        :returns: One :class:`~ecl.network.v2.aws.AWSService`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_aws.AWSService, aws_service)

    def find_aws_service(self, name_or_id, ignore_missing=False):
        """Find a single aws_service

        :param name_or_id: The name or ID of a aws_service.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.aws.AWSService` or None
        """
        return self._find(_aws.AWSService, name_or_id,
                          ignore_missing=ignore_missing)

    def aws_gateways(self, **params):
        """Return a list of aws_gateways
        :param params: The parameters as query string
                       to get aws_gateways by specified condition.
        :returns: A list of aws_gateways objects
        :rtype: list of :class:`~ecl.network.v2.aws.AWSGateway`
        """
        return list(self._list(_aws.AWSGateway, paginated=False, **params))

    def get_aws_gateway(self, aws_gateway):
        """Get a single aws_gateway

        :param aws_gateway:
            The value can be the ID of a aws_gateway or a
            :class:`~ecl.network.v2.aws.AWSGateway` instance.

        :returns: One :class:`~ecl.network.v2.aws.AWSGateway`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_aws.AWSGateway, aws_gateway)

    def find_aws_gateway(self, name_or_id, ignore_missing=False):
        """Find a single aws_gateway

        :param name_or_id: The name or ID of a aws_gateway.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.aws.AWSGateway` or None
        """
        return self._find(_aws.AWSGateway, name_or_id,
                          ignore_missing=ignore_missing)

    def aws_interfaces(self, **params):
        """Return a list of aws_interfaces

        :param params: The parameters as query string
                       to get aws_interfaces by specified condition.
        :returns: A list of aws_interfaces objects
        :rtype: list of :class:`~ecl.network.v2.aws.AWSInterface`
        """
        return list(self._list(_aws.AWSInterface, paginated=False, **params))

    def get_aws_interface(self, aws_interface):
        """Get a single aws_interface

        :param aws_interface:
            The value can be the ID of a aws_interface or a
            :class:`~ecl.network.v2.aws.AWSInterface` instance.

        :returns: One :class:`~ecl.network.v2.aws.AWSInterface`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_aws.AWSInterface, aws_interface)

    def find_aws_interface(self, name_or_id, ignore_missing=False):
        """Find a single aws_interface

        :param name_or_id: The name or ID of a aws_interface.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.network.v2.aws.AWSInterface` or None
        """
        return self._find(_aws.AWSInterface, name_or_id,
                          ignore_missing=ignore_missing)

    def tenant_connections(self, **query):
        """Return a list of tenant_connections

        :param query:  Query parameters to select results.
        :return: A list of tenant_connection objects
        :rtype: list of :class:`~ecl.network.v2.tenant_connection.TenantConnection`
        """
        return list(self._list(_tenant_connection.TenantConnection,
                               paginated=False, **query))

    def get_tenant_connection(self, tenant_connection):
        """Get a single tenant_connection

        :param tenant_connection: The value can be the ID of a TenantConnection
                                  or a :class:`~ecl.network.v2.tenant_connection.TenantConnection`
                                  instance.

        :returns: One :class:`~ecl.network.v2.tenant_connection.TenantConnection`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_tenant_connection.TenantConnection, tenant_connection)

    def create_tenant_connection(self,
                                 connected_network_id, connected_network_tenant_id,
                                 connected_interface_id, connected_interface_ip_address,
                                 interface_type,
                                 name=None, description=None, tenant_id=None,
                                 connected_interface_virtual_ip_address=None,
                                 connected_interface_virtual_ip_vrid=None):
        """Create a TenantConnection from attributes

        :param uuid connected_network_id: ID of the connected network
        :param uuid connected_network_tenant_id: Tenant ID of the Connected Network
        :param uuid connected_interface_id:
                Resource id of the Interface which Connects the Tenant
        :param IPv4 connected_interface_ip_address:
                IP Address of the Interface which Connects the Tenant
        :param IPv4 connected_interface_virtual_ip_address:
                Virtual IP Address of the Interface which Connects the Tenant
        :param integer connected_interface_virtual_ip_vrid:
                VRID of the Interface which Connects the Tenant
        :param string interface_type: Interface Type of the Tenant Connection
        :param string name: Name of the Tenant Connection
        :param string description: Description of the Tenant Connection
        :param uuid tenant_id: Tenant ID of the owner

        :return: results of TenantConnection creation
        :rtype: :class:`~ecl.network.v2.tenant_connection.TenantConnection`
        """
        body = dict()
        connected_interface = dict()
        connected_interface["id"] = connected_interface_id
        connected_interface["ip_address"] = connected_interface_ip_address
        body["connected_interface"] = connected_interface

        connected_network = dict()
        connected_network["id"] = connected_network_id
        connected_network["tenant_id"] = connected_network_tenant_id
        body["connected_network"] = connected_network

        body["interface_type"] = interface_type
        if description:
            body["description"] = description
        if name:
            body["name"] = name
        if tenant_id:
            body["tenant_id"] = tenant_id
        if connected_interface_virtual_ip_address:
            connected_interface["virtual_ip_address"] = \
                connected_interface_virtual_ip_address
        if connected_interface_virtual_ip_vrid:
            connected_interface["virtual_ip_properties"] = \
                {"protocol": "vrrp",
                 "vrid": connected_interface_virtual_ip_vrid}
        return self._create(_tenant_connection.TenantConnection,
                            **body)

    def update_tenant_connection(self, tenant_connection, name=None,
                                 description=None):
        """Update a Tenant Connection from attributes

        :param tenant_connection: The value can be the ID
            of a tenant_connection or
            a :class:`~ecl.network.v2.tenant_connection.TenantConeection` instance.
        :param string name: Name of TenantConnection to update
        :param string description: Description of TenantConnection to update

        :return: results of TenantConnection update
        :rtype: :class:`~ecl.network.v2.tenant_connection.TenantConnection`
        """
        body = dict()
        if name is not None:
            body["name"] = name
        if description is not None:
            body["description"] = description
        if not isinstance(tenant_connection, _tenant_connection.TenantConnection):
            tenant_connection = self._get_resource(
                _tenant_connection.TenantConnection, tenant_connection)
            tenant_connection._body.clean()
        return self._update(_tenant_connection.TenantConnection,
                            tenant_connection, **body)

    def delete_tenant_connection(self, tenant_connection):
        """Delete a Tenant Connection

        :param tenant_connection: The value can be the ID
            of a tenant_connection or
            a :class:`~ecl.network.v2.tenant_connection.TenantConeection` instance.
        :return:
        """
        return self._delete(_tenant_connection.TenantConnection, tenant_connection)

    def execute_tenant_connection(self, tenant_connection):
        if not isinstance(tenant_connection, _tenant_connection.TenantConnection):
            tenant_connection = self._get_resource(
                _tenant_connection.TenantConnection, tenant_connection)
        return tenant_connection.execute(self.session)

    def gcp_services(self, **params):
        """Return a list of gcp_services

        :param params: The parameters as query string
                       to get gcp_services by specified condition.

        :returns: A list of gcp_services objects
        :rtype: list of :class:`~ecl.network.v2.gcp.GCPService`
        """
        return list(self._list(_gcp.GCPService, paginated=False, **params))

    def get_gcp_service(self, gcp_service):
        """Get a single gcp_service

        :param gcp_service:
            The value can be the ID of a gcp_service or a
            :class:`~ecl.network.v2.gcp.GCPService` instance.

        :returns: One :class:`~ecl.network.v2.gcp.GCPService`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_gcp.GCPService, gcp_service)

    def find_gcp_service(self, name_or_id, ignore_missing=False):
        """Find a single gcp_service

        :param name_or_id: The name or ID of a gcp_service.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.

        :returns: One :class:`~ecl.network.v2.gcp.GCPService` or None
        """
        return self._find(_gcp.GCPService, name_or_id,
                          ignore_missing=ignore_missing)

    def gcp_gateways(self, **params):
        """Return a list of gcp_gateways

        :param params: The parameters as query string
                       to get gcp_gateways by specified condition.

        :returns: A list of gcp_gateways objects
        :rtype: list of :class:`~ecl.network.v2.gcp.GCPGateway`
        """
        return list(self._list(_gcp.GCPGateway, paginated=False, **params))

    def get_gcp_gateway(self, gcp_gateway):
        """Get a single gcp_gateway

        :param gcp_gateway:
            The value can be the ID of a gcp_gateway or a
            :class:`~ecl.network.v2.gcp.GCPGateway` instance.


        :returns: One :class:`~ecl.network.v2.gcp.GCPGateway`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_gcp.GCPGateway, gcp_gateway)

    def find_gcp_gateway(self, name_or_id, ignore_missing=False):
        """Find a single gcp_gateway

        :param name_or_id: The name or ID of a gcp_gateway.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.

        :returns: One :class:`~ecl.network.v2.gcp.GCPGateway` or None
        """
        return self._find(_gcp.GCPGateway, name_or_id,
                          ignore_missing=ignore_missing)

    def gcp_interfaces(self, **params):
        """Return a list of gcp_interfaces

        :param params: The parameters as query string
                       to get gcp_interfaces by specified condition.

        :returns: A list of gcp_interfaces objects
        :rtype: list of :class:`~ecl.network.v2.gcp.GCPInterface`
        """
        return list(self._list(_gcp.GCPInterface, paginated=False, **params))

    def get_gcp_interface(self, gcp_interface):
        """Get a single gcp_interface

        :param gcp_interface:
            The value can be the ID of a gcp_interface or a
            :class:`~ecl.network.v2.gcp.GCPInterface` instance.

        :returns: One :class:`~ecl.network.v2.gcp.GCPInterface`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_gcp.GCPInterface, gcp_interface)

    def find_gcp_interface(self, name_or_id, ignore_missing=False):
        """Find a single gcp_interface

        :param name_or_id: The name or ID of a gcp_interface.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.

        :returns: One :class:`~ecl.network.v2.gcp.GCPInterface` or None
        """
        return self._find(_gcp.GCPInterface, name_or_id,
                          ignore_missing=ignore_missing)

    def azure_services(self, **params):
        """Return a list of azure_services

        :param params: The parameters as query string
                       to get azure_services by specified condition.

        :returns: A list of azure_services objects
        :rtype: list of :class:`~ecl.network.v2.azure.AzureService`
        """
        return list(self._list(_azure.AzureService, paginated=False, **params))

    def get_azure_service(self, azure_service):
        """Get a single azure_service

        :param azure_service:
            The value can be the ID of a azure_service or a
            :class:`~ecl.network.v2.azure.AzureService` instance.

        :returns: One :class:`~ecl.network.v2.azure.AzureService`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_azure.AzureService, azure_service)

    def find_azure_service(self, name_or_id, ignore_missing=False):
        """Find a single azure_service

        :param name_or_id: The name or ID of a azure_service.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.

        :returns: One :class:`~ecl.network.v2.azure.AzureService` or None
        """
        return self._find(_azure.AzureService, name_or_id,
                          ignore_missing=ignore_missing)

    def azure_gateways(self, **params):
        """Return a list of azure_gateways

        :param params: The parameters as query string
                       to get azure_gateways by specified condition.

        :returns: A list of azure_gateways objects
        :rtype: list of :class:`~ecl.network.v2.azure.AzureGateway`
        """
        return list(self._list(_azure.AzureGateway, paginated=False, **params))

    def get_azure_gateway(self, azure_gateway):
        """Get a single azure_gateway

        :param azure_gateway:
            The value can be the ID of a azure_gateway or a
            :class:`~ecl.network.v2.azure.AzureGateway` instance.

        :returns: One :class:`~ecl.network.v2.azure.AzureGateway`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_azure.AzureGateway, azure_gateway)

    def find_azure_gateway(self, name_or_id, ignore_missing=False):
        """Find a single azure_gateway

        :param name_or_id: The name or ID of a azure_gateway.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.

        :returns: One :class:`~ecl.network.v2.azure.AzureGateway` or None
        """
        return self._find(_azure.AzureGateway, name_or_id,
                          ignore_missing=ignore_missing)

    def azure_interfaces(self, **params):
        """Return a list of azure_interfaces

        :param params: The parameters as query string
                       to get azure_interfaces by specified condition.

        :returns: A list of azure_interfaces objects
        :rtype: list of :class:`~ecl.network.v2.azure.AzureInterface`
        """
        return list(self._list(_azure.AzureInterface, paginated=False, **params))

    def get_azure_interface(self, azure_interface):
        """Get a single azure_interface

        :param azure_interface:
            The value can be the ID of a azure_interface or a
            :class:`~ecl.network.v2.azure.AzureInterface` instance.

        :returns: One :class:`~ecl.network.v2.azure.AzureInterface`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_azure.AzureInterface, azure_interface)

    def find_azure_interface(self, name_or_id, ignore_missing=False):
        """Find a single azure_interface

        :param name_or_id: The name or ID of a azure_interface.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.

        :returns: One :class:`~ecl.network.v2.azure.AzureInterface` or None
        """
        return self._find(_azure.AzureInterface, name_or_id,
                          ignore_missing=ignore_missing)



# -*- coding: utf-8 -*-


from ecl.network import network_service
from ecl.network.v2.base import NetworkBaseResource
from ecl import resource2
from ecl import utils


class TenantConnection(NetworkBaseResource):
    resource_key = "tenant_connection"
    resources_key = "tenant_connections"

    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/tenant_connections'

    allow_list = True
    allow_get = True
    allow_create = True
    allow_delete = True
    allow_update = True

    _query_mapping = resource2.QueryParameters(
        "approval_request_id", "interface_type",
        "name", "description", "id", "progress",
        "tenant_id", "sort_key", "sort_dir",
    )

    approval_request_id = resource2.Body("approval_request_id")
    connected_interface = resource2.Body("connected_interface", dict)
    connected_network = resource2.Body("connected_network", dict)
    interface_type = resource2.Body("interface_type")
    description = resource2.Body("description")
    id = resource2.Body("id")
    name = resource2.Body("name")
    progress = resource2.Body("progress")
    tenant_id = resource2.Body("tenant_id")

    def execute(self, session):
        """Preform tenant connection execute."""
        url = utils.urljoin(TenantConnection.base_path, self.id, 'execute')
        headers = {'Accept': ''}
        return session.post(
            url, endpoint_filter=self.service, headers=headers)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest
import uuid

from ecl.tests.functional import base


@unittest.skipUnless(base.service_exists(service_type="metering"),
                     "Metering service does not exist")
class TestMeter(base.BaseFunctionalTest):

    def test_list(self):
        # TODO(thowe): Remove this in favor of create_meter call.
        # Since we do not have a create meter method at the moment
        # make sure there is some data in there
        name = uuid.uuid4().hex
        tainer = self.conn.object_store.create_container(name=name)
        self.conn.object_store.delete_container(tainer)

        names = set([o.name for o in self.conn.telemetry.meters()])
        self.assertIn('storage.objects.incoming.bytes', names)



# -*- coding: utf-8 -*-

import six

from ecl.tests.functional import base


class TestExtension(base.BaseFunctionalTest):

    def test_list(self):
        extensions = self.conn.network.extensions()
        self.assertGreaterEqual(len(extensions), 0)

        for ext in extensions:
            self.assertIsInstance(ext.name, six.string_types)
            self.assertIsInstance(ext.alias, six.string_types)

    def test_find(self):
        extension = self.conn.network.find_extension('external-net')
        self.assertEqual('Neutron external network', extension.name)



# -*- coding: utf-8 -*-

from ecl.compute import compute_service
from ecl import resource2


class ServerAction(resource2.Resource):
    resource_key = 'instanceAction'
    resources_key = 'instanceActions'
    base_path = '/servers/%(instance_uuid)s/os-instance-actions'
    service = compute_service.ComputeService()

    # capabilities
    allow_list = True
    allow_get = True

    # Properties
    #: action instance
    action = resource2.Body('action')
    #: The ID for the server.
    instance_uuid = resource2.URI('instance_uuid')
    #: Message of action
    message = resource2.Body('message')
    #: 	project id
    project_id = resource2.Body('project_id')
    #: request id
    request_id = resource2.Body('request_id', alternate_id=True)
    #: start time of an action
    start_time = resource2.Body('start_time')
    #: owner of the instance
    user_id = resource2.Body('user_id')



# Copyright 2010 Jacob Kaplan-Moss
# Copyright 2011 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Exception definitions.
"""

import six
import json


class SDKException(Exception):
    """The base exception class for all exceptions this library raises."""
    def __init__(self, message=None, cause=None):
        self.message = self.__class__.__name__ if message is None else message
        self.cause = cause
        super(SDKException, self).__init__(self.message)


class EndpointNotFound(SDKException):
    """A mismatch occurred between what the client and server expect."""
    def __init__(self, message=None):
        super(EndpointNotFound, self).__init__(message)


class InvalidResponse(SDKException):
    """The response from the server is not valid for this request."""

    def __init__(self, response):
        super(InvalidResponse, self).__init__()
        self.response = response


class InvalidRequest(SDKException):
    """The request to the server is not valid."""

    def __init__(self, message=None):
        super(InvalidRequest, self).__init__(message)


class HttpException(SDKException):

    def __init__(self, message=None, details=None, response=None,
                 request_id=None, url=None, method=None,
                 http_status=None, cause=None):
        super(HttpException, self).__init__(message=message, cause=cause)
        self.details = details
        self.response = response
        self.request_id = request_id
        self.url = url
        self.method = method
        self.http_status = http_status
        if self.response is not None:
            self.message = self._get_exception_message(message=self.message)

    def _is_compute_form_error(self, content):
        try:
            if 'message' in content.items()[0][1]:
                return True
        except:
            return False

    def _get_exception_message(self, message=None):
        try:
            content = json.loads(self.response._content)

            if self._is_compute_form_error(content):
                return content.items()[0][1]['message']

            # API-GW
            if 'fault' in content and 'faultstring' in content['fault']:
                return content['fault']['faultstring']

            # IAM error case.
            if content.get('errorMessage'):
                return content[self.api_error_key]

            # Function specific case.
            if hasattr(self, 'api_error_key') \
                    and content.get(self.api_error_key):
                return content[self.api_error_key]

            # Default case.
            if content.get('message'):
                return content['message']
        except:
            pass

        if message:
            return message

        # In case parse failed.
        return 'Unknown Exception'

    def __unicode__(self):
        msg = self.__class__.__name__ + ": " + self.message
        if self.details:
            msg += ", " + six.text_type(self.details)
        return msg

    def __str__(self):
        return self.__unicode__()


class NotFoundException(HttpException):
    """HTTP 404 Not Found."""
    pass


class MethodNotSupported(SDKException):
    """The resource does not support this operation type."""
    def __init__(self, resource, method):
        # This needs to work with both classes and instances.
        try:
            name = resource.__name__
        except AttributeError:
            name = resource.__class__.__name__

        message = ('The %s method is not supported for %s.%s' %
                   (method, resource.__module__, name))
        super(MethodNotSupported, self).__init__(message=message)


class DuplicateResource(SDKException):
    """More than one resource exists with that name."""
    pass


class ResourceNotFound(NotFoundException):
    """No resource exists with that name or id."""
    pass


class ResourceTimeout(SDKException):
    """Timeout waiting for resource."""
    pass


class ResourceFailure(SDKException):
    """General resource failure."""
    pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import six
from ecl.tests.functional import base


class TestServer(base.BaseFunctionalTest):

    @classmethod
    def test_01_create(cls):
        server = cls.conn.baremetal.create_server(
            server = {
                "name":"SDK-TEST-BARE-02",
                "flavorRef":"462f4f62-0b0a-4171-acfc-a3840fd50b4b",
                "imageRef":"4fc19fa6-4643-44bd-a52d-5388d4d011b1",
                "networks":[{"uuid":"4fdac559-914e-41b2-9e58-e5a36ba437dd"}]
            }
        )
        cls.server_id = server.id
        print server
        assert isinstance(server.id, six.string_types)

    def test_02_list(self):
        servers = list(self.conn.baremetal.servers(
            limit="10",
        ))
        server = servers[0]
        print server
        self.assertIsInstance(server.id, six.string_types)

    def test_03_list_detail(self):
        servers = list(self.conn.baremetal.servers(details=True))
        server = servers[0]
        print server
        self.assertIsInstance(server.id, six.string_types)

    def test_04_show_server(self):
        server = self.conn.baremetal.get_server("752aac2e-4b82-4d47-a7c7-fcbd0cbc86e2")
        print server
        self.assertIsInstance(server.OS_EXT_STS_power_state, six.string_types)
        self.assertIsInstance(server.OS_EXT_STS_task_state, six.string_types)
        self.assertIsInstance(server.OS_EXT_STS_vm_state, six.string_types)
        self.assertIsInstance(server.OS_EXT_AZ_availability_zone, six.string_types)
        self.assertIsInstance(server.created, six.string_types)
        self.assertIsInstance(server.flavor, dict)
        #self.assertIsInstance(server.hostId, six.string_types)
        self.assertIsInstance(server.image, dict)
        self.assertIsInstance(server.metadata, dict)
        self.assertIsInstance(server.links, list)
        self.assertIsInstance(server.progress, int)
        self.assertIsInstance(server.status, six.string_types)
        self.assertIsInstance(server.tenant_id, six.string_types)
        self.assertIsInstance(server.updated, six.string_types)
        self.assertIsInstance(server.user_id, six.string_types)
        self.assertIsInstance(server.raid_arrays, list)
        #self.assertIsInstance(server.lvm_volume_groups, list)
        #self.assertIsInstance(server.filesystems, list)
        self.assertIsInstance(server.nic_physical_ports, list)
        self.assertIsInstance(server.chassis_status, dict)

    def test_05_delete_server(self):
        server = self.conn.baremetal.delete_server("752aac2e-4b82-4d47-a7c7-xx")
        assert False



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl import exceptions
from ecl.orchestration.v1 import resource as _resource
from ecl.orchestration.v1 import software_config as _sc
from ecl.orchestration.v1 import software_deployment as _sd
from ecl.orchestration.v1 import stack as _stack
from ecl.orchestration.v1 import template as _template
from ecl import proxy2


class Proxy(proxy2.BaseProxy):

    def create_stack(self, preview=False, **attrs):
        """Create a new stack from attributes

        :param bool perview: When ``True``, returns
            an :class:`~ecl.orchestration.v1.stack.StackPreview` object,
            otherwise an :class:`~ecl.orchestration.v1.stack.Stack`
            object.
            *Default: ``False``*
        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~ecl.orchestration.v1.stack.Stack`,
                           comprised of the properties on the Stack class.

        :returns: The results of stack creation
        :rtype: :class:`~ecl.orchestration.v1.stack.Stack`
        """
        res_type = _stack.StackPreview if preview else _stack.Stack
        return self._create(res_type, **attrs)

    def find_stack(self, name_or_id, ignore_missing=False):
        """Find a single stack

        :param name_or_id: The name or ID of a stack.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.orchestration.v1.stack.Stack` or None
        """
        return self._find(_stack.Stack, name_or_id,
                          ignore_missing=ignore_missing)

    def stacks(self, **query):
        """Return a generator of stacks

        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A list of stack objects
        :rtype: :class:`~ecl.orchestration.v1.stack.Stack`
        """
        return list(self._list(_stack.Stack, paginated=False, **query))

    def get_stack(self, stack):
        """Get a single stack

        :param stack: The value can be the ID of a stack or a
               :class:`~ecl.orchestration.v1.stack.Stack` instance.

        :returns: One :class:`~ecl.orchestration.v1.stack.Stack`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_stack.Stack, stack)

    def update_stack(self, stack, **attrs):
        """Update a stack

        :param stack: The value can be the ID of a stack or a
               :class:`~ecl.orchestration.v1.stack.Stack` instance.
        :param kwargs \*\*attrs: The attributes to update on the stack
                                 represented by ``value``.

        :returns: The updated stack
        :rtype: :class:`~ecl.orchestration.v1.stack.Stack`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._update(_stack.Stack, stack, **attrs)

    def delete_stack(self, stack, ignore_missing=False):
        """Delete a stack

        :param stack: The value can be either the ID of a stack or a
                      :class:`~ecl.orchestration.v1.stack.Stack`
                      instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the stack does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent stack.

        :returns: ``None``
        """
        self._delete(_stack.Stack, stack, ignore_missing=ignore_missing)

    def abandon_stack(self, stack, ignore_missing=False):
        """Abandon a stack

        :param stack: The value can be either the ID of a stack or a
                      :class:`~ecl.orchestration.v1.stack.Stack`
                      instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the stack does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to abandon a nonexistent stack.
        :return: ``None``
        """
        try:
            if not isinstance(stack, _stack.Stack):
                stack = self.get_stack(stack)
            stack.abandon(self.session)
        except exceptions.NotFoundException as e:
            if ignore_missing:
                return None
            else:
                # Reraise with a more specific type and message
                raise exceptions.ResourceNotFound(
                    message="No %s found for %s" %
                            (_stack.Stack.__name__, str(stack)),
                    details=e.details, response=e.response,
                    request_id=e.request_id, url=e.url, method=e.method,
                    http_status=e.http_status, cause=e.cause)

    def check_stack(self, stack):
        """Check a stack's status

        Since this is an asynchronous action, the only way to check the result
        is to track the stack's status.

        :param stack: The value can be either the ID of a stack or an instance
                      of :class:`~ecl.orchestration.v1.stack.Stack`.
        :returns: ``None``
        """
        if isinstance(stack, _stack.Stack):
            stk_obj = stack
        else:
            stk_obj = _stack.Stack.existing(id=stack)

        stk_obj.check(self.session)

    def resources(self, stack, **query):
        """Return a generator of resources

        :param stack: This can be a stack object, or the name of a stack
                      for which the resources are to be listed.
        :param kwargs \*\*query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A list of resource objects if the stack exists and
                  there are resources in it. If the stack cannot be found,
                  an exception is thrown.
        :rtype: A generator of
            :class:`~ecl.orchestration.v1.resource.Resource`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when the stack cannot be found.
        """
        # first try treat the value as a stack object or an ID
        if isinstance(stack, _stack.Stack):
            obj = stack
        else:
            obj = self._find(_stack.Stack, stack, ignore_missing=False)

        return list(self._list(_resource.Resource, paginated=False,
                          stack_name=obj.name, stack_id=obj.id, **query))

    def create_software_config(self, **attrs):
        """Create a new software config from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~ecl.orchestration.v1.software_config.SoftwareConfig`,
            comprised of the properties on the SoftwareConfig class.

        :returns: The results of software config creation
        :rtype:
            :class:`~ecl.orchestration.v1.software_config.SoftwareConfig`
        """
        return self._create(_sc.SoftwareConfig, **attrs)

    def software_configs(self, **query):
        """Returns a generator of software configs

        :param dict query: Optional query parameters to be sent to limit the
                           software configs returned.
        :returns: A list of software config objects.
        :rtype:
        :class:`~ecl.orchestration.v1.software_config.SoftwareConfig`
        """
        return list(self._list(_sc.SoftwareConfig, paginated=True, **query))

    def get_software_config(self, software_config):
        """Get details about a specific software config.

        :param software_config: The value can be the ID of a software config
            or a instace of
            :class:`~ecl.orchestration.v1.software_config.SoftwareConfig`,

        :returns: An object of type
            :class:`~ecl.orchestration.v1.software_config.SoftwareConfig`
        """
        return self._get(_sc.SoftwareConfig, software_config)

    def delete_software_config(self, software_config, ignore_missing=False):
        """Delete a software config

        :param software_config: The value can be either the ID of a software
            config or an instance of
            :class:`~ecl.orchestration.v1.software_config.SoftwareConfig`
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the software config does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent software config.
        :returns: ``None``
        """
        self._delete(_sc.SoftwareConfig, software_config,
                     ignore_missing=ignore_missing)

    def create_software_deployment(self, **attrs):
        """Create a new software deployment from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~ecl.orchestration.v1.software_deployment.SoftwareDeployment`,
            comprised of the properties on the SoftwareDeployment class.

        :returns: The results of software deployment creation
        :rtype:
            :class:`~ecl.orchestration.v1.software_deployment.SoftwareDeployment`
        """
        return self._create(_sd.SoftwareDeployment, **attrs)

    def software_deployments(self, **query):
        """Returns a generator of software deployments

        :param dict query: Optional query parameters to be sent to limit the
                           software deployments returned.
        :returns: A generator of software deployment objects.
        :rtype:
        :class:`~ecl.orchestration.v1.software_deployment.SoftwareDeployment`
        """
        return self._list(_sd.SoftwareDeployment, paginated=False, **query)

    def get_software_deployment(self, software_deployment):
        """Get details about a specific software deployment resource

        :param software_deployment: The value can be the ID of a software
            deployment or an instace of
            :class:`~ecl.orchestration.v1.software_deployment.SoftwareDeployment`,

        :returns: An object of type
            :class:`~ecl.orchestration.v1.software_deployment.SoftwareDeployment`
        """
        return self._get(_sd.SoftwareDeployment, software_deployment)

    def delete_software_deployment(self, software_deployment,
                                   ignore_missing=False):
        """Delete a software deployment

        :param software_deployment: The value can be either the ID of a
            software deployment or an instance of
            :class:`~ecl.orchestration.v1.software_deployment.SoftwareDeployment`
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the software deployment does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent software deployment.
        :returns: ``None``
        """
        self._delete(_sd.SoftwareDeployment, software_deployment,
                     ignore_missing=ignore_missing)

    def update_software_deployment(self, software_deployment, **attrs):
        """Update a software deployment

        :param server: Either the ID of a software deployment or an instance of
            :class:`~ecl.orchestration.v1.software_deployment.SoftwareDeployment`
        :param dict attrs: The attributes to update on the software deployment
                           represented by ``software_deployment``.

        :returns: The updated software deployment
        :rtype:
        :class:`~ecl.orchestration.v1.software_deployment.SoftwareDeployment`
        """
        return self._update(_sd.SoftwareDeployment, software_deployment,
                            **attrs)

    def validate_template(self, template, environment=None, template_url=None,
                          ignore_errors=None):
        """Validates a template.

        :param template: The stack template on which the validation is
                         performed.
        :param environment: A JSON environment for the stack, if provided.
        :param template_url: A URI to the location containing the stack
                             template for validation. This parameter is only
                             required if the ``template`` parameter is None.
                             This parameter is ignored if ``template`` is
                             specified.
        :param ignore_errors: A string containing comma separated error codes
                              to ignore. Currently the only valid error code
                              is '99001'.
        :returns: The result of template validation.
        :raises: :class:`~ecl.exceptions.InvalidRequest` if neither
                 `template` not `template_url` is provided.
        :raises: :class:`~ecl.exceptions.HttpException` if the template
                 fails the validation.
        """
        if template is None and template_url is None:
            raise exceptions.InvalidRequest(
                "'template_url' must be specified when template is None")

        tmpl = _template.Template.new()
        return tmpl.validate(self.session, template, environment=environment,
                             template_url=template_url,
                             ignore_errors=ignore_errors)



# -*- coding: utf-8 -*-


from ecl.connectivity.v1.cic import CIC, EIC
from ecl.connectivity.v1.mcic import MCIC, MEIC
from ecl.connectivity.v1.operation import Operation
from ecl.connectivity.v1.server_segment import ServerSegment

from ecl import proxy2


class Proxy(proxy2.BaseProxy):

    def mcics(self, **query):
        """Return a list of MCICs

        :param kwargs query: Query parameter to get mCICs.
            * string tenant_id: ID of a tenant to retrieve mCICs.

        :returns: A list of MCICs objects
        """
        mcic_list = self._list(MCIC, paginated=False, **query)
        return [mcic for mcic in mcic_list if mcic.service_type == "NGC-colo"]

    def meics(self, **query):
        """Return a list of MEICs

        :param kwargs query: Query parameter to get mEICs.
            * string tenant_id: ID of a tenant to retrieve mEICs.

        :returns: A list of MEICs objects
        """
        meic_list = self._list(MEIC, paginated=False, **query)
        return [meic for meic in meic_list if meic.service_type == "NGC-EC"]

    def get_mcic(self, mcic):
        """Get a single mcic

        :param mcic: The value can be the ID of a mcic or a
                       :class:`~ecl.connectivity.v2.mcic.MCIC` instance.

        :returns: One :class:`~ecl.connectivity.v2.mcic.MCIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(MCIC, mcic)

    def get_meic(self, meic):
        """Get a single meic

        :param meic: The value can be the ID of a meic or a
                       :class:`~ecl.connectivity.v2.mcic.MEIC` instance.

        :returns: One :class:`~ecl.connectivity.v2.mcic.MEIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(MEIC, meic)

    def cics(self, mcic_id, **query):
        """Return a list of CICs

        :param string mcic_id: ID of a mcic that cics belong to

        :returns: A list of CICs objects
        """
        return self._list(CIC, paginated=False, mcic_id=mcic_id, **query)

    def eics(self, meic_id, **query):
        """Return a list of EICs

        :param string meic_id: ID of a meic that eics belong to

        :returns: A list of EICs objects
        """
        return self._list(EIC, paginated=False, mcic_id=meic_id, **query)

    def get_cic(self, mcic_id, cic):
        """Get a single cic

        :param string mcic_id: ID of a mcic
        :param cic: The value can be the ID of a cic or a
                       :class:`~ecl.connectivity.v2.cic.CIC` instance.

        :returns: One :class:`~ecl.connectivity.v2.cic.CIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        cic = self._get_resource(CIC, cic, mcic_id=mcic_id)
        return self._get(CIC, cic)

    def get_eic(self, meic_id, eic):
        """Get a single eic

        :param string meic_id: ID of a meic
        :param eic: The value can be the ID of a eic or a
                       :class:`~ecl.connectivity.v2.cic.EIC` instance.

        :returns: One :class:`~ecl.connectivity.v2.cic.EIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        eic = self._get_resource(EIC, eic, mcic_id=meic_id)
        return self._get(CIC, eic)

    def create_cic(self, mcic_id, cic_name, network_id, colo_vlan):
        """Create a single cic

        :param string mcic_id: The value is the ID of a mcic
        :param string cic_name: The value is the name of a cic
        :param string network_id: The value is the ID of network of a cic
        :param int colo_vlan: The value indicates the colocation vlan of a cic

        :returns: One :class:`~ecl.connectivity.v2.cic.CIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        body = {
            "cic_name": cic_name,
            "network_id": network_id,
            "colo_vlan": colo_vlan,
            "mcic_id": mcic_id
        }
        return self._create(CIC, **body)

    def create_eic(self, meic_id, eic_name, network_id, server_segment_nbr):
        """Create a single eic

        :param string meic_id: The value is the ID of a meic
        :param string eic_name: The value is the name of  eic


        :returns: One :class:`~ecl.connectivity.v2.cic.EIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        body = {
            "cic_name": eic_name,
            "network_id": network_id,
            "server_segment_nbr": server_segment_nbr,
            "mcic_id": meic_id
        }
        return self._create(EIC, **body)

    def delete_cic(self, mcic_id, cic):
        """Delete a single cic

        :param string mcic_id: The value is the ID of a mcic
        :param cic: The value can be the ID of a cic or a
                       :class:`~ecl.connectivity.v2.cic.CIC` instance.

        :returns: One :class:`~ecl.connectivity.v2.cic.CIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._delete(CIC, cic, mcic_id=mcic_id)

    def delete_eic(self, meic_id, eic):
        """Delete a single eic

        :param string meic_id: The value is the ID of a meic
        :param eic: The value can be the ID of a eic or a
                       :class:`~ecl.connectivity.v2.cic.EIC` instance.

        :returns: One :class:`~ecl.connectivity.v2.cic.EIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._delete(EIC, eic, mcic_id=meic_id)

    def update_cic(self, mcic_id, cic, cic_name):
        """Update a single cic

        :param string mcic_id: The value is the ID of a mcic
        :param cic: The value can be the ID of a cic or a
                       :class:`~ecl.connectivity.v2.cic.CIC` instance.
        :param cic_name: The name of CIC to update

        :returns: One :class:`~ecl.connectivity.v2.cic.CIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        body = {"cic_name": cic_name}
        res = self._get_resource(CIC, cic, **body)
        return res.update(self.session, mcic_id, cic, **body)

    def update_eic(self, meic_id, eic, eic_name):
        """Update a single eic

        :param string meic_id: The value is the ID of a meic
        :param eic: The value can be the ID of a eic or a
                       :class:`~ecl.connectivity.v2.cic.EIC` instance.
        :param eic_name: The name of EIC to update

        :returns: One :class:`~ecl.connectivity.v2.cic.EIC`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        body = {"cic_name": eic_name}
        res = self._get_resource(CIC, eic, **body)
        return res.update(self.session, meic_id, eic, **body)

    def operations(self, **query):
        """Return a list of MCICs

        :param kwargs query: Query parameter to get operations.
            * string mcic_id: ID of a mcic
            * string cic_id: ID of a cic

        :returns: A list of Operation objects
        """
        return list(self._list(Operation, paginated=False, **query))

    def get_operation(self, operation):
        """Get a single operation

        :param operation: The value can be the ID of a operation or a
                       :class:`~ecl.connectivity.v2.operation.Operation` instance.

        :returns: One :class:`~ecl.connectivity.v2.operation.Operation`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(Operation, operation)

    def server_segments(self, service_id):
        """Return a list of Server Segments.
        :param string service_id: Enterprise Cloud service ID
        :returns: A list of ServerSegment objects
        """
        return list(self._list(ServerSegment, service_id=service_id))



# -*- coding: utf-8 -*-


from ecl import service_filter


class ConnectivityService(service_filter.ServiceFilter):
    """The interconnectivity service."""

    valid_versions = [service_filter.ValidVersion('v1')]

    def __init__(self, version=None):
        """Create a interconnectivity service."""
        super(ConnectivityService, self).__init__(service_type='interconnectivity',
                                                  version=version)



# -*- coding: utf-8 -*-

from ecl import utils
from ecl.connectivity import connectivity_service
from ecl import resource2


class CIC(resource2.Resource):

    resource_key = None
    resources_key = None
    base_path = '/mCICs/%(mcic_id)s/CICs'
    service = connectivity_service.ConnectivityService()

    # capabilities
    allow_get = True
    allow_list = True
    allow_create = True
    allow_delete = True
    allow_update = True

    _query_mapping = resource2.QueryParameters("sort_key", "sort_dir")

    cic_id = resource2.Body("cic_id", alternate_id=True)
    cic_name = resource2.Body("cic_name")
    cic_status = resource2.Body("cic_status")
    network_id = resource2.Body("network_id")
    network_name = resource2.Body("network_name")
    destination_vlan = resource2.Body("destination_vlan", type=int)
    colo_vlan = resource2.Body("colo_vlan", type=int)
    bandwidth = resource2.Body("bandwidth", type=int)
    mcic_id = resource2.URI("mcic_id")

    def update(self, session, mcic_id, cic_id, has_body=True, **attrs):
        uri = self.base_path % ({'mcic_id': mcic_id})
        uri = utils.urljoin(uri, cic_id)
        args = {'json': attrs}
        resp = session.put(uri, endpoint_filter=self.service, **args)
        self._translate_response(resp, has_body=False)
        return self

class EIC(CIC):
    cic_id = resource2.Body("cic_id", alternate_id=True)
    server_segment_nbr = resource2.Body("server_segment_nbr",
                                        type=int)
    server_segment_name = resource2.Body("server_segment_name")
    destination_vlan = None
    colo_vlan = None


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.identity import identity_service
from ecl import resource


class Group(resource.Resource):
    resource_key = 'group'
    resources_key = 'groups'
    base_path = '/groups'
    service = identity_service.IdentityService()

    # capabilities
    allow_create = True
    allow_retrieve = True
    allow_update = True
    allow_delete = True
    allow_list = True
    patch_update = True

    # Properties
    #: The description of this group. *Type: string*
    description = resource.prop('description')
    #: References the domain ID which owns the group; if a domain ID is not
    #: specified by the client, the Identity service implementation will
    #: default it to the domain ID to which the client's token is scoped.
    #: *Type: string*
    domain_id = resource.prop('domain_id')
    #: Unique group name, within the owning domain. *Type: string*
    name = resource.prop('name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import urllib
from ecl.dedicated_hypervisor import dedicated_hypervisor_service
from ecl import resource2


class Usage(resource2.Resource):
    resources_key = "usages"
    resource_key = None
    base_path = '/usages'
    service = dedicated_hypervisor_service.DedicatedHypervisorService()

    # Capabilities
    allow_list = True
    allow_get = True

    # Properties
    #: history id of Guest Image usage.
    id = resource2.Body('id')
    #: license type name for the usage.
    name = resource2.Body('name')
    #: license type description for the usage.
    description = resource2.Body('description')
    #: type of Guest Image usage.
    type = resource2.Body('type')
    #: if true, there is license key.
    has_license_key = resource2.Body('has_license_key')
    #: unit for the usage.
    unit = resource2.Body('unit')
    #: usage value.
    value = resource2.Body('value')
    #: uuid for the resource.
    resource_id = resource2.Body('resource_id')
    #: date to list usage from.
    From = resource2.Body('from')
    #: date to list usage to.
    #: month of the parameter must be the same as `from` .
    to = resource2.Body('to')
    #: name of license type to show.
    license_type = resource2.Body('license_type')
    #: histories of the usage
    histories = resource2.Body('histories')

    def get_usage_histories(self, session, history_id, **kwargs):
        """Shows your Guest Image usage history information."""

        params = []
        for key in kwargs.keys():
            item = "%s=%s" % (str(key), urllib.quote(str(kwargs[key]))) \
                if key in ['from', 'to'] \
                else "%s=%s" % (str(key), str(kwargs[key]))
            params.append(item)

        uri = self.base_path + '/%s/histories' % history_id
        if len(params) != 0:
            uri += '?'
            for item in params:
                uri += item + '&'
            uri = uri[:-1]

        resp = session.get(uri, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self



# -*- coding: utf-8 -*-


import six

from ecl import exceptions
from ecl.tests.functional import base


class TestMCIC(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestMCIC, cls).setUpClass()
        mcics = list(cls.conn.connectivity.mcics())
        cls.one_mcic = None
        if mcics and len(mcics) > 0:
            cls.one_mcic = mcics[0]

    def test_mcics(self):
        mcics = list(self.conn.connectivity.mcics())
        self.assertGreaterEqual(len(mcics), 0)

        for mcic in mcics:
            # print mcic.to_dict()["service_type"]
            self.assertIsInstance(mcic.mcic_id, six.string_types)
            self.assertIsInstance(mcic.mcic_name, six.string_types)
            self.assertIsInstance(mcic.mcic_status, six.string_types)
            self.assertIsInstance(mcic.tenant_id, six.string_types)
            self.assertIsInstance(mcic.tenant_name, six.string_types)
            self.assertEqual(mcic.service_type, "NGC-colo")
            if mcic.ngc:
                self.assertIsInstance(mcic.ngc, dict)
            if mcic.colo:
                self.assertIsInstance(mcic.colo, dict)
            self.assertIsInstance(mcic.timezone, six.string_types)

    def test_get_mcic_by_id(self):
        mcic = self.conn.connectivity.get_mcic(self.one_mcic.mcic_id)
        self.assertEqual(mcic.mcic_id, self.one_mcic.mcic_id)
        self.assertEqual(mcic.mcic_name, self.one_mcic.mcic_name)
        self.assertEqual(mcic.mcic_status, self.one_mcic.mcic_status)
        self.assertEqual(mcic.tenant_id, self.one_mcic.tenant_id)
        self.assertEqual(mcic.tenant_name, self.one_mcic.tenant_name)
        self.assertEqual(mcic.service_type, self.one_mcic.service_type)
        self.assertEqual(mcic.ngc, self.one_mcic.ngc)
        self.assertEqual(mcic.colo, self.one_mcic.colo)
        self.assertEqual(mcic.timezone, self.one_mcic.timezone)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.rca import rca_service
from ecl import resource2

class Version(resource2.Resource):
    resource_key = 'version'
    resources_key = 'versions'
    base_path = ''
    service = rca_service.RcaService(
        version=rca_service.RcaService.UNVERSIONED
    )

    # capabilities
    allow_list = True
    allow_get = True

    # Properties
    #: Version identifier included in API URL.
    id = resource2.Body('id')
    #: List of API endpoint link.
    links = resource2.Body('links')
    #: Version support status. Valid values are CURRENT or SUPPORTED.
    #: CURRENT is newest stable version. SUPPORTED is old supported version.
    status = resource2.Body('status')

    def get_version(self, session):
        url = self.base_path + '/v1.0'
        resp = session.get(
            url, headers={"Accept": "application/json"}, endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self

    def list_version(self, session):
        url = self.base_path
        resp = session.get(
            url, headers={"Accept": "application/json"}, endpoint_filter=self.service
        )
        resp = resp.json()[self.resources_key]

        for data in resp:
            version = self.existing(**data)
            yield version


# -*- coding: utf-8 -*-

import base

from ecl.virtual_network_appliance import virtual_network_appliance_service
from ecl import resource2


class Operation(base.VirtualNetworkApplianceBaseResource):
    resources_key = "operations"
    resource_key = "operation"
    service = virtual_network_appliance_service.\
        VirtualNetworkApplianceService("v1.0")
    base_path = '/' + service.version + '/operations'

    # Capabilities
    allow_list = True
    allow_get = True

    # Properties
    #: ID of network appliance's operation
    id = resource2.Body('id')
    #: Target network appliance id of operation.
    resource_id = resource2.Body('resource_id')
    #: request type of operation.
    request_type = resource2.Body('request_type')
    #: Status of operation.
    status = resource2.Body('status')
    #: Reception datetime of operation.
    reception_datetime = resource2.Body('reception_datetime')
    #: Commit datetime of operation.
    commit_datetime = resource2.Body('commit_datetime')
    #: Request body(JSON String) of operation.
    request_body = resource2.Body('request_body')
    #: Warning of operation.
    warning = resource2.Body('warning')
    #: Error of operation.
    error = resource2.Body('error')
    #: Tenant ID of operation.
    tenant_id = resource2.Body('tenant_id')
    #: resource type of operation.
    resource_type = resource2.Body('resource_type')

    @classmethod
    def list(cls, session, paginated=False, resource_ids=[],
             no_deleted=True, latest=False):
        """This method is a generator which yields operation objects.

        This resource object list generator handles pagination and takes query
        params for response filtering.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param bool paginated: ``True`` if a GET to this resource returns
            a paginated series of responses, or ``False``
            if a GET returns only one page of data.
            **When paginated is False only one
            page of data will be returned regardless
            of the API's support of pagination.**
        :param list resource_ids: These arguments are passed as
            resource_id query_string.
        :param bool no_deleted:
            ``True`` You can get operations for only existing resources.
            ``False`` You can get operations for all resources even removed.
        :param bool latest:
            ``True`` You can get only the latest operations for each resource.
            ``False`` You can get all of operations for each resource.
        :return: A generator of :class:`Operation` objects.
        :raises: :exc:`~ecl.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_list` is not set to ``True``.
        """
        more_data = True

        uri = \
            cls.base_path + \
            '?no_deleted=%s&latest=%s' % (str(no_deleted).lower(),
                                          str(latest).lower())

        list_for_resource_ids = []
        if resource_ids:
            for resource_id in resource_ids:
                list_for_resource_ids.append('resource_id=%s' % resource_id)

        resource_id_query = '&'.join(list_for_resource_ids)
        if len(list_for_resource_ids) > 0:
            uri += '&%s' % resource_id_query

        while more_data:
            resp = session.get(uri, endpoint_filter=cls.service,
                               headers={"Accept": "application/json"})
            resp = resp.json()
            if cls.resources_key:
                resp = resp[cls.resources_key]

            if not resp:
                more_data = False

            # Keep track of how many items we've yielded. If we yielded
            # less than our limit, we don't need to do an extra request
            # to get back an empty data set, which acts as a sentinel.
            yielded = 0
            new_marker = None
            for data in resp:
                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                data.pop("self", None)

                value = cls.existing(**data)
                new_marker = value.id
                yielded += 1
                yield value

            if not paginated:
                return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.baremetal import baremetal_service
from ecl import resource2


class UEFI(resource2.Resource):
    resource_key = 'uefi'
    resources_key = 'uefi'
    base_path = '/servers/%(server_id)s/uefi'
    service = baremetal_service.BaremetalService()

    # capabilities
    allow_update = True
    allow_get = True

    #: Name for the flavor.
    flavor_name = resource2.Body('flavor_name')
    #: The ID for the flavor.
    flavor_id = resource2.Body('flavor_id', alternate_id=True)
    #: Updated time for UEFI
    updated = resource2.Body('updated')
    #: The last result of applying uefi setting.
    status = resource2.Body('status')
    #: Error messages when updating uefi setting is failed.
    message = resource2.Body('message')
    #: UEFI setting.
    setting = resource2.Body('setting', type=dict)

    def get(self, session, server_id):
        uri = self.base_path % {"server_id": server_id}
        resp = session.get(uri, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self

    def update(self, session, server_id, has_body=True, **attrs):
        uri = self.base_path % {"server_id": server_id}
        body = attrs
        args = {'json': body}
        resp = session.put(uri, endpoint_filter=self.service, **args)
        self._translate_response(resp, has_body=False)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestLoadBalancerInterface(base.BaseFunctionalTest):
    def test_01_load_balancer_interfaces(self):
        load_balancer_interfaces = list(self.conn.network.load_balancer_interfaces(

        ))
        print load_balancer_interfaces

    def test_02_show_load_balancer_interface(self):
        interface = self.conn.network.show_load_balancer_interface(
            "0b2d4f07-313c-4bee-a1dd-4982ccef9bb1"
        )
        print interface
        #self.assertIsInstance(interface.description, six.string_types)
        self.assertIsInstance(interface.id, six.string_types)
        #self.assertIsInstance(interface.ip_address, six.string_types)
        self.assertIsInstance(interface.load_balancer_id, six.string_types)
        self.assertIsInstance(interface.name, six.string_types)
        #self.assertIsInstance(interface.network_id, six.string_types)
        #self.assertIsInstance(interface.slot_number, six.string_types)
        self.assertIsInstance(interface.status, six.string_types)
        self.assertIsInstance(interface.tenant_id, six.string_types)
        #self.assertIsInstance(interface.virtual_ip_address, six.string_types)
        #self.assertIsInstance(interface.virtual_ip_properties, dict)

    def test_03_update_load_balancer_interface(self):
        interface = self.conn.network.update_load_balancer_interface(
            "0b2d4f07-313c-4bee-a1dd-4982ccef9bb1",
            description="updated"
        )
        print interface
        self.assertIsInstance(interface.description, six.string_types)


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.storage.v1 import volume as v

IDENTIFIER = 'IDENTIFIER'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    'name': 'vol01',
    'description': 'Volume for vs01',
    'status': 'creating',
    'size': 100,
    'iops_per_gb': 4,
    'initiator_iqns': [],
    'initiator_secret': 'null',
    'target_secret': 'null',
    'snapshot_ids': [],
    'target_ips': [],
    'metadata': {},
    'virtual_storage_id': '2',
    'availability_zone': '3',
    'created_at': 'null',
    'updated_at': 'null',
    'error_message': '',
    'throughput': 50,
    'export_rules': []
}


class TestVolume(testtools.TestCase):

    def test_basic(self):
        sot = v.Volume()
        self.assertEqual('volume', sot.resource_key)
        self.assertEqual('volumes', sot.resources_key)
        self.assertEqual('/volumes', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('storage', sot.service.service_type)

    def test_basic_detail(self):
        sot = v.VolumeDetail()
        self.assertEqual('volume', sot.resource_key)
        self.assertEqual('volumes', sot.resources_key)
        self.assertEqual('/volumes/detail', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertEqual('storage', sot.service.service_type)

    def test_make_basic(self):
        sot = v.Volume(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['name'], sot.name)
        self.assertEqual(BASIC_EXAMPLE['description'], sot.description)
        self.assertEqual(BASIC_EXAMPLE['status'], sot.status)
        self.assertEqual(BASIC_EXAMPLE['size'], sot.size)
        self.assertEqual(BASIC_EXAMPLE['iops_per_gb'], sot.iops_per_gb)
        self.assertEqual(BASIC_EXAMPLE['initiator_iqns'], sot.initiator_iqns)
        self.assertEqual(BASIC_EXAMPLE['initiator_secret'], sot.initiator_secret)
        self.assertEqual(BASIC_EXAMPLE['target_secret'], sot.target_secret)
        self.assertEqual(BASIC_EXAMPLE['snapshot_ids'], sot.snapshot_ids)
        self.assertEqual(BASIC_EXAMPLE['target_ips'], sot.target_ips)
        self.assertEqual(BASIC_EXAMPLE['metadata'], sot.metadata)
        self.assertEqual(BASIC_EXAMPLE['virtual_storage_id'], sot.virtual_storage_id)
        self.assertEqual(BASIC_EXAMPLE['availability_zone'], sot.availability_zone)
        self.assertEqual(BASIC_EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(BASIC_EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(BASIC_EXAMPLE['error_message'], sot.error_message)
        self.assertEqual(BASIC_EXAMPLE['throughput'], sot.throughput)
        self.assertEqual(BASIC_EXAMPLE['export_rules'], sot.export_rules)
